@@ -1,11 +1,11 @@
 module Example(fac, main) where
 
-fac :: Int -> Int
+fac :: Integer -> Integer
 fac 0 = 1
 fac n = n * fac(n - 1)
 
 main :: IO ()
 main = do
-  let rs = map fac [1,2,3,10]
+  let rs = map fac [1,2,3,10,52]
   putStrLn "Some factorials"
   print rs
