@@ -1,7 +1,8 @@
 module Control.Category(module Control.Category) where
-import Prelude()
+import qualified Prelude()
 import Primitives
 import qualified Data.Function as F
+import {-# SOURCE #-} Data.Typeable
 
 infixr 9 .
 infixr 1 >>>, <<<
