@@ -32,9 +32,11 @@ module Control.Monad(
   liftM,
   liftM2,
   liftM3,
+  liftM4,
+  liftM5,
   ap,
   ) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives  -- for fixity
 import Control.Applicative
 import Control.Error
@@ -46,6 +48,7 @@ import Data.List
 import Data.Monoid.Internal
 import Data.Ord
 --import Data.Maybe
+import {-# SOURCE #-} Data.Typeable
 
 infixl 1 >>, >>=, =<<
 infixr 1 <=<, >=>
@@ -53,7 +56,7 @@ infixr 1 <=<, >=>
 class (Applicative m) => Monad m where
   (>>=)  :: forall a b . m a -> (a -> m b) -> m b
   (>>)   :: forall a b . m a -> m b -> m b
-  ma >> mb = ma >>= \ _ -> mb
+  ma >> mb = ma >>= const mb
 
   -- Maybe remove this
   return :: forall a . a -> m a
@@ -133,7 +136,7 @@ foldM :: forall m a b . (Monad m) => (b -> a -> m b) -> b -> [a] -> m b
 foldM = foldlM
 
 foldM_ :: forall m a b . (Monad m) => (b -> a -> m b) -> b -> [a] -> m ()
-foldM_ f a xs  = foldlM f a xs >> return ()
+foldM_ f a xs  = void (foldlM f a xs)
 
 foldlM :: forall m a b . (Monad m) => (b -> a -> m b) -> b -> [a] -> m b
 foldlM _ z [] = pure z
@@ -153,7 +156,7 @@ replicateM_ cnt0 f = loop cnt0
   where
     loop cnt =
       if cnt <= (0::Int) then pure ()
-      else f *> (loop (cnt `primIntSub` 1))
+      else f *> loop (cnt `primIntSub` 1)
 
 -----
 
@@ -166,11 +169,15 @@ unless p ma = if p then pure () else ma
 -----
 
 liftM :: forall m r a1 . (Monad m) => (a1 -> r) -> m a1 -> m r
-liftM f m1 = do { x1 <- m1; return (f x1) }
+liftM f m1 = f <$> m1
 liftM2 :: forall m r a1 a2 . (Monad m) => (a1 -> a2 -> r) -> m a1 -> m a2 -> m r
 liftM2 f m1 m2 = do { x1 <- m1; x2 <- m2; return (f x1 x2) }
 liftM3 :: forall m r a1 a2 a3 . (Monad m) => (a1 -> a2 -> a3 -> r) -> m a1 -> m a2 -> m a3 -> m r
 liftM3 f m1 m2 m3 = do { x1 <- m1; x2 <- m2; x3 <- m3; return (f x1 x2 x3) }
+liftM4 :: forall m r a1 a2 a3 a4 . (Monad m) => (a1 -> a2 -> a3 -> a4 -> r) -> m a1 -> m a2 -> m a3 -> m a4 -> m r
+liftM4 f m1 m2 m3 m4 = do { x1 <- m1; x2 <- m2; x3 <- m3; x4 <- m4; return (f x1 x2 x3 x4) }
+liftM5 :: forall m r a1 a2 a3 a4 a5 . (Monad m) => (a1 -> a2 -> a3 -> a4 -> a5 -> r) -> m a1 -> m a2 -> m a3 -> m a4 -> m a5 -> m r
+liftM5 f m1 m2 m3 m4 m5 = do { x1 <- m1; x2 <- m2; x3 <- m3; x4 <- m4; x5 <- m5; return (f x1 x2 x3 x4 x5) }
 
 ap :: forall m a b . Monad m => m (a -> b) -> m a -> m b
 ap f a = do
@@ -180,14 +187,14 @@ ap f a = do
 
 -----
 
-instance forall a . Functor ((->) a) where
+instance Functor ((->) a) where
   fmap = (.)
 
-instance forall a . Applicative ((->) a) where
+instance Applicative ((->) a) where
   pure = const
   f <*> g = \ a -> f a (g a)
 
-instance forall a . Monad ((->) a) where
+instance Monad ((->) a) where
   x >>= y = \ z -> y (x z) z
 
 instance Monad Dual where
@@ -222,8 +229,7 @@ instance MonadPlus [] where
   mplus = (++)
 
 msum :: forall m a . (MonadPlus m) => [m a] -> m a
-msum [] = mzero
-msum (ma:mas) = ma `mplus` msum mas
+msum = foldr mplus mzero
 
 mfilter :: forall m a . (MonadPlus m) => (a -> Bool) -> m a -> m a
 mfilter p ma = do
