@@ -1,11 +1,12 @@
 module Control.Monad.Fail(MonadFail(..)) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Control.Applicative
 import Control.Error
 import Control.Monad
 import Data.Char
 import Data.List_Type
+import {-# SOURCE #-} Data.Typeable
 
 class Monad m => MonadFail m where
   fail :: forall a . String -> m a
