@@ -1,9 +1,10 @@
 module Control.Monad.IO.Class (MonadIO(..)) where
-import Prelude()
+import qualified Prelude()
 import Data.Char
 import Control.Applicative
 import Control.Monad
 import System.IO
+import {-# SOURCE #-} Data.Typeable
 
 class (Monad m) => MonadIO m where
   liftIO :: IO a -> m a
