@@ -4,7 +4,7 @@ module Data.Bool(
   module Data.Bool,
   module Data.Bool_Type
   ) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Bool_Type
 import Data.Bounded
