@@ -1,6 +1,7 @@
 module Data.Bounded(module Data.Bounded) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
+import {-# SOURCE #-} Data.Typeable
 
 class Bounded a where
   minBound :: a
