@@ -1,5 +1,5 @@
 module Data.Char_Type(Char, String) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.List_Type
 
