@@ -1,16 +1,24 @@
-module Data.Complex(module Data.Complex) where
+module Data.Complex (
+  Complex((:+)),
+  realPart,
+  imagPart,
+  mkPolar,
+  cis,
+  polar,
+  magnitude,
+  phase,
+  conjugate,
+) where
+
 import Data.Typeable
+import Text.ParserCombinators.ReadPrec
+import Text.Read.Internal
+import qualified Text.Read.Lex as L
 
 infix 6 :+
 
 data Complex a = !a :+ !a
-  deriving(Typeable)
-
-instance forall a . Eq a => Eq (Complex a) where
-  (x :+ y) == (x' :+ y')  =  x == x' && y == y'   -- parser bug
-
-instance forall a . Show a => Show (Complex a) where
-  show (x :+ y) = show x ++ " :+ " ++ show y
+  deriving (Typeable, Eq, Read, Show)
 
 realPart :: forall a . Complex a -> a
 realPart (x :+ _) =  x
@@ -41,23 +49,21 @@ magnitude (x:+y) =
   in  mx * sqrt(1 + r*r)
 
 phase :: forall a . (RealFloat a) => Complex a -> a
--- XXX phase (0 :+ 0)   = 0
-phase (x:+y) | x==0 && y==0 = 0
-             | otherwise    = atan2 y x
+phase (0 :+ 0) = 0
+phase (x :+ y) = atan2 y x
 
 
-instance forall a . (RealFloat a) => Num (Complex a)  where
+instance (RealFloat a) => Num (Complex a)  where
     (x:+y) + (x':+y')   =  (x+x') :+ (y+y')
     (x:+y) - (x':+y')   =  (x-x') :+ (y-y')
     (x:+y) * (x':+y')   =  (x*x'-y*y') :+ (x*y'+y*x')
     negate (x:+y)       =  negate x :+ negate y
     abs z               =  magnitude z :+ 0
---    signum (0:+0)       =  0
-    signum z@(x:+y) | x==0 && y==0 = 0
-                    | otherwise    =  x/r :+ y/r  where r = magnitude z
+    signum (0:+0)       =  0
+    signum z@(x:+y)     =  x/r :+ y/r  where r = magnitude z
     fromInteger n       =  fromInteger n :+ 0
 
-instance forall a . (RealFloat a) => Fractional (Complex a)  where
+instance (RealFloat a) => Fractional (Complex a)  where
     (x:+y) / (x':+y')   =  (x*x''+y*y'') / d :+ (y*x''-x*y'') / d
                            where x'' = scaleFloat k x'
                                  y'' = scaleFloat k y'
@@ -66,39 +72,36 @@ instance forall a . (RealFloat a) => Fractional (Complex a)  where
 
     fromRational a      =  fromRational a :+ 0
 
-instance forall a . (RealFloat a) => Floating (Complex a) where
+instance (RealFloat a) => Floating (Complex a) where
     pi             =  pi :+ 0
     exp (x:+y)     =  expx * cos y :+ expx * sin y
                       where expx = exp x
     log z          =  log (magnitude z) :+ phase z
-{-
-    x ** y = case (x,y) of
- {- XXX
-     (_ , (0:+0))  -> 1 :+ 0
-      ((0:+0), (exp_re:+_)) -> case compare exp_re 0 of
-                 GT -> 0 :+ 0
-                 LT -> inf :+ 0
-                 EQ -> nan :+ nan
--}
-      ((re:+im), (exp_re:+_))
-        | (isInfinite re || isInfinite im) -> case compare exp_re 0 of
-                 GT -> inf :+ 0
-                 LT -> 0 :+ 0
-                 EQ -> nan :+ nan
+
+    x ** y = case (x, y) of
+      (_ , 0 :+ 0) -> 1 :+ 0
+      (0 :+ 0, exp_re :+ _) -> case compare exp_re 0 of
+        GT -> 0 :+ 0
+        LT -> inf :+ 0
+        EQ -> nan :+ nan
+      (re :+ im, exp_re :+ _)
+        | isInfinite re || isInfinite im -> case compare exp_re 0 of
+            GT -> inf :+ 0
+            LT -> 0 :+ 0
+            EQ -> nan :+ nan
         | otherwise -> exp (log x * y)
      where
-        inf = 1/0
-        nan = 0/0
--}
-    sqrt z@(x:+y) | x==0 && y==0 = 0
-                  | otherwise    =  u :+ (if y < 0 then - v else v)
+        inf = 1 / 0
+        nan = 0 / 0
+    sqrt (0:+0)    =  0
+    sqrt z@(x:+y)  =  u :+ (if y < 0 then - v else v)
                       where (u,v) = if x < 0 then (v',u') else (u',v')
                             v'    = abs y / (u'*2)
                             u'    = sqrt ((magnitude z + abs x) / 2)
 
     sin (x:+y)     =  sin x * cosh y :+ cos x * sinh y
-    cos (x:+y)     =  cos x * cosh y :+ (- (sin x) * sinh y)
-    tan (x:+y)     =  (sinx*coshy:+cosx*sinhy)/(cosx*coshy:+(- sinx*sinhy))
+    cos (x:+y)     =  cos x * cosh y :+ (- (sin x * sinh y))
+    tan (x:+y)     =  (sinx*coshy :+ cosx*sinhy)/(cosx*coshy :+ (- (sinx*sinhy)))
                       where sinx  = sin x
                             cosx  = cos x
                             sinhy = sinh y
@@ -122,8 +125,8 @@ instance forall a . (RealFloat a) => Floating (Complex a) where
 
     asinh z        =  log (z + sqrt (1+z*z))
 
-    acosh z        =  log (z + (sqrt $ z+1) * (sqrt $ z - 1))
-    atanh z        =  0.5 * log ((1.0+z) / (1.0-z))
+    acosh z        =  log (z + sqrt (z + 1) * sqrt (z - 1))
+    atanh z        =  0.5 * log ((1 + z) / (1 - z))
 
     log1p x@(a :+ b)
       | abs a < 0.5 && abs b < 0.5
@@ -134,5 +137,5 @@ instance forall a . (RealFloat a) => Floating (Complex a) where
       | a*a + b*b < 1
       , u <- expm1 a
       , v <- sin (b/2)
-      , w <- -2*v*v = (u*w + u + w) :+ (u+1)*sin b
+      , w <- -(2*v*v) = (u*w + u + w) :+ (u+1)*sin b
       | otherwise = exp x - 1
