@@ -1,8 +1,8 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module Data.Constraint(module Data.Constraint) where
-import Prelude()              -- do not import Prelude
-import Primitives  -- for (->)
+import qualified Prelude()              -- do not import Prelude
+import {-# SOURCE #-} Data.Typeable
 import Text.Show
 
 -- A very, very minimal version of the constraints package
@@ -10,7 +10,7 @@ import Text.Show
 type Dict :: Constraint -> Type
 data Dict c = c => Dict
 
-instance forall c . Show (Dict c) where
+instance Show (Dict c) where
   showsPrec _ Dict = showString "Dict"
 
 withDict :: forall c r . Dict c -> (c => r) -> r
