@@ -11,8 +11,15 @@ import Data.Maybe
 import Data.Proxy
 import Data.Typeable
 import Unsafe.Coerce
+import Control.Exception
 
 data Dynamic = D TypeRep AnyType
+  deriving Typeable
+
+instance Exception Dynamic
+
+instance Show Dynamic where
+  show (D t _) = "<<" ++ show t ++ ">>"
 
 toDyn :: forall a . Typeable a => a -> Dynamic
 toDyn a = D (typeOf a) (unsafeCoerce a)
@@ -28,8 +35,8 @@ dynApp :: Dynamic -> Dynamic -> Dynamic
 dynApp f a = fromMaybe (error "Dynamic.dynApp") $ dynApply f a
 
 dynApply :: Dynamic -> Dynamic -> Maybe Dynamic
-dynApply (D ftr f) (D atr a) = fmap g $ funResultTy ftr atr
-  where g rtr = D rtr ((unsafeCoerce f) a)
+dynApply (D ftr f) (D atr a) = g <$> funResultTy ftr atr
+  where g rtr = D rtr (unsafeCoerce f a)
 
 dynTypeRep :: Dynamic -> TypeRep
 dynTypeRep (D tr _) = tr
