@@ -1,7 +1,7 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module Data.Function(module Data.Function) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Bool_Type
 --import Data.Tuple
@@ -37,6 +37,7 @@ on op sel x y = op (sel x) (sel y)
 asTypeOf :: forall a . a -> a -> a
 asTypeOf = const
 
+infixr 0 `seq`
 seq :: forall a b . a -> b -> b
 seq = primSeq
 
