@@ -1,7 +1,7 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module Data.Functor.Const(Const(..), getConst) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Control.Applicative
 import Control.Monad
@@ -10,7 +10,7 @@ import Data.Eq
 import Data.Function
 import Data.Functor
 import Data.Functor.Const_Type
-import Data.Int
+import Data.Int.Int
 import Data.Monoid.Internal
 import Data.Ord
 import Text.Show
