@@ -1,18 +1,24 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module Data.Integral(module Data.Integral) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Control.Error
 import Data.Bool
+import Data.Enum_Class
 import Data.Eq
 import Data.Integer_Type
 import Data.Num
 import Data.Ord
+import Data.Real
+import {-# SOURCE #-} Data.Typeable
 
 infixl 7 `quot`,`rem`
+infixl 7 `div`,`mod`
 
-class {-(Real a, Enum a) => -} (Eq a, Num a) => Integral a where
+-- XXX Importing Data.Enum causes an import cycle.
+--     I don't really see the point of Enum as a superclass.
+class (Real a, Enum a) => Integral a where
   quot      :: a -> a -> a
   rem       :: a -> a -> a
   div       :: a -> a -> a
@@ -38,7 +44,7 @@ lcm x y =
   if x == 0 || y == 0 then
     0
   else
-    abs ((x `quot` (gcd x y)) * y)
+    abs ((x `quot` gcd x y) * y)
 
 even :: forall a . (Integral a) => a -> Bool
 even n = n `rem` 2 == 0
