@@ -98,7 +98,7 @@ module Data.List.NonEmpty (
    , nonEmpty    -- :: [a] -> Maybe (NonEmpty a)
    , xor         -- :: NonEmpty Bool -> Bool
    ) where
-import Prelude()
+import qualified Prelude()
 
 import Control.Applicative
 import Control.Error
@@ -109,7 +109,7 @@ import Data.Foldable hiding (length, toList)
 import qualified Data.Foldable as Foldable
 import Data.Function
 import Data.Functor hiding(unzip)
-import Data.Int
+import Data.Int.Int
 import qualified Data.List as List
 import Data.List_Type
 import Data.List.NonEmpty_Type
@@ -119,7 +119,6 @@ import Data.Monoid.Internal
 import Data.Ord
 import Data.Traversable
 import Data.Tuple
-import GHC.Stack(HasCallStack)
 import Text.Show
 
 {- In Data.List.NonEmpty_Type
@@ -129,11 +128,8 @@ data NonEmpty a = a :| [a]
   deriving (Eq, Ord)
 -}
 
-instance (Eq a) => Eq (NonEmpty a) where
-  (x :| xs) == (y :| ys)  =  x == y && xs == ys
-
-instance (Ord a) => Ord (NonEmpty a) where
-  compare (x :| xs) (y :| ys) = compare x y <> compare xs ys
+deriving instance (Eq a) => Eq (NonEmpty a)
+deriving instance (Ord a) => Ord (NonEmpty a)
 
 instance Semigroup (NonEmpty a) where
   (a :| as) <> ~(b :| bs) = a :| (as ++ b : bs)
@@ -151,19 +147,20 @@ instance Monad NonEmpty where
   ~(a :| as) >>= f = b :| (bs ++ bs')
     where b :| bs = f a
           bs' = as >>= toList . f
-          
+
 --instance Foldable NonEmpty -- Defined in Data.Foldable
 --instance Traversable NonEmpty -- Defined in Data.Traversable
 --instance Read a => Read (NonEmpty a) -- Defined in GHC.Read
 --instance Show a => Show (NonEmpty a) -- Defined in GHC.Show
 
-instance Show a => Show (NonEmpty a) where
-  showsPrec p = showsPrec p . toList
+deriving instance Show a => Show (NonEmpty a)
+
 instance Foldable NonEmpty where
   foldr f z = foldr f z . toList
---instance Traversable NonEmpty where
---  traverse f = fromList . traverse f . toList
-  
+
+instance Traversable NonEmpty where
+  traverse f = fmap fromList . traverse f . toList
+
 ----- End MHS replacement
 
 infixr 5 <|
@@ -282,7 +279,7 @@ sortOn f = lift (List.sortOn f) . toList
 -- | Converts a normal list to a 'NonEmpty' stream.
 --
 -- Raises an error if given an empty list.
-fromList :: HasCallStack => [a] -> NonEmpty a
+fromList :: {-HasCallStack => -} [a] -> NonEmpty a
 fromList (a:as) = a :| as
 fromList [] = error "NonEmpty.fromList: empty list"
 
@@ -585,7 +582,7 @@ isPrefixOf (y:ys) (x :| xs) = (y == x) && List.isPrefixOf ys xs
 -- @n@. Note that the head of the stream has index 0.
 --
 -- /Beware/: a negative or out-of-bounds index will cause an error.
-(!!) :: HasCallStack => NonEmpty a -> Int -> a
+(!!) :: {-HasCallStack => -} NonEmpty a -> Int -> a
 (!!) ~(x :| xs) n
   | n == 0 = x
   | n > 0  = xs List.!! (n - 1)
@@ -620,7 +617,7 @@ nub = nubBy (==)
 -- user-supplied equality predicate instead of the overloaded '=='
 -- function.
 nubBy :: (a -> a -> Bool) -> NonEmpty a -> NonEmpty a
-nubBy eq (a :| as) = a :| List.nubBy eq (List.filter (\b -> not (eq a b)) as)
+nubBy eq (a :| as) = a :| List.nubBy eq (List.filter (not . eq a) as)
 
 -- | 'transpose' for 'NonEmpty', behaves the same as 'GHC.Internal.Data.List.transpose'
 -- The rows/columns need not be the same length, in which case
