@@ -1,5 +1,6 @@
 module Data.List.NonEmpty_Type(module Data.List.NonEmpty_Type) where
-import Prelude()
+import qualified Prelude()
+import {-# SOURCE #-} Data.Typeable
 
 infixr 5 :|
 
