@@ -1,7 +1,7 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module Data.Maybe(module Data.Maybe, module Data.Maybe_Type) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Control.Applicative
 import Control.Error
@@ -12,25 +12,25 @@ import Data.Char
 import Data.Eq
 import Data.Function
 import Data.Functor
-import Data.Int
+import Data.Int.Int
 import Data.List
 import Data.Maybe_Type
 import Data.Monoid.Internal
 import Data.Ord
 import Text.Show
 
-instance forall a . Eq a => Eq (Maybe a) where
+instance Eq a => Eq (Maybe a) where
   Nothing == Nothing  =  True
   Just x  == Just x'  =  x == x'
   _       == _        =  False
 
-instance forall a . Ord a => Ord (Maybe a) where
+instance Ord a => Ord (Maybe a) where
   Nothing `compare` Nothing = EQ
   Nothing `compare` Just _  = LT
   Just _  `compare` Nothing = GT
   Just x  `compare` Just y  = x `compare` y
 
-instance forall a . (Show a) => Show (Maybe a) where
+instance (Show a) => Show (Maybe a) where
   showsPrec _ Nothing  = showString "Nothing"
   showsPrec p (Just a) = showParen (p >= 11) (showString "Just " . showsPrec 11 a)
 
@@ -61,12 +61,12 @@ instance Alternative Maybe where
   Nothing <|> y = y
   x       <|> _ = x
 
-instance forall a . Semigroup a => Semigroup (Maybe a) where
+instance Semigroup a => Semigroup (Maybe a) where
   Nothing <> b       = b
   a       <> Nothing = a
   Just a  <> Just b  = Just (a <> b)
 
-instance forall a . Semigroup a => Monoid (Maybe a) where
+instance Semigroup a => Monoid (Maybe a) where
   mempty = Nothing
 
 maybe :: forall a r . r -> (a -> r) -> Maybe a -> r
