@@ -1,9 +1,10 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module Data.Num(module Data.Num) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Integer_Type
+import {-# SOURCE #-} Data.Typeable
 
 infixl 6 +,-
 infixl 7 *
