@@ -1,18 +1,19 @@
 module Data.Proxy(module Data.Proxy) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Bool_Type
 import Data.Eq
 import Data.Functor
+import {-# SOURCE #-} Data.Typeable
 import Text.Show
 
 type Proxy :: forall (k::Kind) . k -> Type
 data Proxy a = Proxy
 
-instance forall a . Show (Proxy a) where
+instance Show (Proxy a) where
   show _ = "Proxy"
 
-instance forall a . Eq (Proxy a) where
+instance Eq (Proxy a) where
   _ == _  =  True
 
 instance Functor Proxy where
