@@ -5,10 +5,11 @@ module Data.Ratio(
   approxRational,
   rationalInfinity,
   rationalNaN,
-  rationalMinusZero,
+  rationalNegativeZero,
   ) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Data.Bool
+import Data.Enum
 import Data.Eq
 import Data.Fractional
 import Data.Function
@@ -38,21 +39,31 @@ data Ratio a = !a :% !a
 --  (x + y) * z + 0*z == x*z + y*z
 -- When z is a normal rational number the is clearly the same
 -- as the usual distributive law.
--- 
+--
 -- NOTE. Experimentally, we extend this with:
 --  x/0 ==  1/0 when x > 0
 --  x/0 == -1/0 when x < 0
 
-instance forall a . Eq a => Eq (Ratio a) where
+instance (Integral a) => Enum (Ratio a) where
+  succ x         = x + 1
+  pred x         = x - 1
+  toEnum         = fromIntegral
+  fromEnum       = fromInteger . truncate
+  enumFrom       = numericEnumFrom
+  enumFromThen   = numericEnumFromThen
+  enumFromTo     = numericEnumFromTo
+  enumFromThenTo = numericEnumFromThenTo
+
+instance Eq a => Eq (Ratio a) where
   (x :% y) == (x' :% y')  =  x == x' && y == y'
 
-instance forall a . (Integral a, Ord a) => Ord (Ratio a) where
+instance (Integral a, Ord a) => Ord (Ratio a) where
   (x :% y) <= (x' :% y')  =  x * y' <= x' * y
   (x :% y) <  (x' :% y')  =  x * y' <  x' * y
   (x :% y) >= (x' :% y')  =  x * y' >= x' * y
   (x :% y) >  (x' :% y')  =  x * y' >  x' * y
 
-instance forall a . (Integral a, Ord a) => Num (Ratio a) where
+instance (Integral a, Ord a) => Num (Ratio a) where
   (x:%y) + (x':%y')   =  reduce (x*y' + x'*y) (y*y')
   (x:%y) - (x':%y')   =  reduce (x*y' - x'*y) (y*y')
   (x:%y) * (x':%y')   =  reduce (x * x') (y * y')
@@ -61,20 +72,20 @@ instance forall a . (Integral a, Ord a) => Num (Ratio a) where
   signum (x:%_)       =  signum x :% 1
   fromInteger x       =  fromInteger x :% 1
 
-instance forall a . (Integral a, Ord a) => Fractional (Ratio a) where
+instance (Integral a, Ord a) => Fractional (Ratio a) where
   (x:%y) / (x':%y')   = reduce (x*y') (y*x')
   recip (x:%y)
     | x < 0           = (-y) :% (-x)
     | otherwise       =  y :%  x
   fromRational (x:%y) =  fromInteger x % fromInteger y
 
-instance forall a . (Show a) => Show (Ratio a)  where
+instance (Show a) => Show (Ratio a)  where
   showsPrec p (x:%y) = showParen (p > 7) $
                        showsPrec 8 x .
                        showString " % " .
                        showsPrec 8 y
 
-instance forall a . (Integral a, Ord a) => Real (Ratio a) where
+instance (Integral a, Ord a) => Real (Ratio a) where
   toRational (x :% y) = toInteger x :% toInteger y
 
 rationalInfinity :: Rational
@@ -83,8 +94,8 @@ rationalInfinity = 1 :% 0
 rationalNaN :: Rational
 rationalNaN = 0 :% 0
 
-rationalMinusZero :: Rational
-rationalMinusZero = 0 :% (-1)
+rationalNegativeZero :: Rational
+rationalNegativeZero = 0 :% (-1)
 
 infixl 7 %
 (%) :: forall a . (Integral a, Ord a) => a -> a -> Ratio a
