@@ -1,7 +1,8 @@
 module Data.Ratio_Type(module Data.Ratio_Type) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Integer_Type
+import {-# SOURCE #-} Data.Typeable
 
 data Ratio a = !a :% !a
 
