@@ -1,16 +1,17 @@
 module Data.RealFloat(
   RealFloat(..),
   ) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Bool
 import Data.Eq
 import Data.Floating
 import Data.Fractional
-import Data.Int
+import Data.Int.Int
 import Data.Integer
 import Data.Num
 import Data.Ord
+import {-# SOURCE #-} Data.Typeable
 
 class (Fractional a, Ord a, Floating a) => RealFloat a  where
   floatRadix          :: a -> Integer
