@@ -5,10 +5,11 @@ module Data.Records(
   hasField,
   composeSet,
   ) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Function
 import Data.Proxy
+import {-# SOURCE #-} Data.Typeable
 
 type Get r a = r -> a
 type Set r a = r -> a -> r
