@@ -4,10 +4,11 @@ module Data.TypeLits(
   KnownNat(..),
   KnownSymbol(..),
   ) where
-import Prelude()
+import qualified Prelude()
 import Primitives
 import Data.Char_Type
 import Data.Integer
+import {-# SOURCE #-} Data.Typeable
 
 class KnownNat (n :: Nat) where
   natVal :: forall (proxy :: Nat -> Type) . proxy n -> Integer
