@@ -1,7 +1,8 @@
 module Data.Void(module Data.Void) where
-import Prelude(); import MiniPrelude
+import qualified Prelude(); import MiniPrelude
+import Text.Read
 
-data Void
+data Void deriving (Eq, Ord, Read, Show)
 
 absurd :: forall a . Void -> a
 absurd v = seq v (error "absurd")
