@@ -1,11 +1,13 @@
 module Debug.Trace(module Debug.Trace) where
-import Prelude(); import MiniPrelude
+import qualified Prelude(); import MiniPrelude
+import System.IO.Base(hPutStrLn, stderr, hFlush)
 import Primitives
 
 trace :: forall a . String -> a -> a
-trace msg a = primitive "IO.performIO" (
+trace msg a = _primitive "IO.performIO" (
   do
-    putStrLn msg
+    hPutStrLn stderr msg
+    hFlush stderr
     return a
   )
 
