@@ -4,9 +4,9 @@ module Foreign
   , module Data.Word
   , module Foreign.Ptr
   , module Foreign.ForeignPtr
---  , module Foreign.StablePtr
-  , module Foreign.Storable
   , module Foreign.Marshal
+  , module Foreign.StablePtr
+  , module Foreign.Storable
   ) where
 
 import Data.Bits
@@ -14,6 +14,6 @@ import Data.Int
 import Data.Word
 import Foreign.Ptr
 import Foreign.ForeignPtr
---import Foreign.StablePtr
-import Foreign.Storable
 import Foreign.Marshal
+import Foreign.StablePtr
+import Foreign.Storable
