@@ -4,8 +4,9 @@ module Foreign.Marshal.Error (
   throwIfNeg,
   throwIfNeg_,
   throwIfNull,
+  void,
 ) where
-import Prelude(); import MiniPrelude
+import qualified Prelude(); import MiniPrelude
 import Data.Functor(void)
 import Foreign.Ptr
 import System.IO.Error(ioError, userError)
