@@ -1,5 +1,5 @@
 module System.IO.TimeMilli(getTimeMilli) where
-import Prelude(); import MiniPrelude
+import qualified Prelude(); import MiniPrelude
 
 foreign import ccall "GETTIMEMILLI" c_getTimeMilli :: IO Int
 
