@@ -1,7 +1,7 @@
 module System.Info(os, arch, compilerName, compilerVersion, fullCompilerVersion) where
 import Control.Monad
 import Data.Char
-import Data.Version(Version(..))
+import Data.Version
 import System.Cmd
 import System.Directory
 import System.Exit
@@ -18,10 +18,7 @@ compilerName :: String
 compilerName = "mhs"
 
 compilerVersion :: Version
-compilerVersion = Version [0,9]
-
-fullCompilerVersion :: Version
-fullCompilerVersion = Version [0,9,0]
+compilerVersion = fullCompilerVersion { versionBranch = take 2 (versionBranch fullCompilerVersion) }
 
 -- Assume the system has an uname command
 uname :: String -> String
@@ -32,5 +29,7 @@ uname flag = unsafePerformIO $ do
   res <- readFile fn
   removeFile fn
   when (rc /= ExitSuccess) $
-    error $ "System.Into: uname failed"
+    error "System.Into: uname failed"
   return $ map toLower $ filter (not . isSpace) res
+
+fullCompilerVersion = makeVersion [0,15,0,0]
