@@ -95,6 +95,7 @@ module Text.Printf(
 import Data.Char
 import Data.Int
 import Data.List (stripPrefix)
+import Data.Maybe
 import Data.Word
 import Numeric.Show
 import Numeric.FormatFloat(showEFloat, showFFloat, showGFloat, showFFloatAlt, showGFloatAlt)
@@ -301,12 +302,12 @@ instance (a ~ ()) => HPrintfType (IO a) where
 
 -- | @since 2.01
 instance (PrintfArg a, PrintfType r) => PrintfType (a -> r) where
-    spr fmts args = \ a -> spr fmts
+    spr fmts args a = spr fmts
                              ((parseFormat a, formatArg a) : args)
 
 -- | @since 2.01
 instance (PrintfArg a, HPrintfType r) => HPrintfType (a -> r) where
-    hspr hdl fmts args = \ a -> hspr hdl fmts
+    hspr hdl fmts args a = hspr hdl fmts
                                   ((parseFormat a, formatArg a) : args)
 
 -- | Typeclass of 'printf'-formattable values. The 'formatArg' method
@@ -395,11 +396,9 @@ instance PrintfArg Natural where
 instance PrintfArg Float where
     formatArg = formatRealFloat
 
-{- Double=Float
 -- | @since 2.01
 instance PrintfArg Double where
     formatArg = formatRealFloat
--}
 
 -- | This class, with only the one instance, is used as
 -- a workaround for the fact that 'String', as a concrete
@@ -495,9 +494,7 @@ parseIntFormat _ s =
   where
     matchPrefix (p, _) m@(Just (FormatParse p0 _ _))
       | length p0 >= length p = m
-      | otherwise = case getFormat p of
-          Nothing -> m
-          Just fp -> Just fp
+      | otherwise = maybe m Just (getFormat p)
     matchPrefix (p, _) Nothing =
       getFormat p
     getFormat p =
@@ -598,7 +595,7 @@ formatIntegral m x ufmt0 =
     'u' -> (adjust ufmt (fmtu 10 Nothing prec m x) ++)
     'c' | x >= fromIntegral (ord (minBound :: Char)) &&
           x <= fromIntegral (ord (maxBound :: Char)) &&
-          fmtPrecision ufmt == Nothing &&
+          isNothing (fmtPrecision ufmt) &&
           fmtModifiers ufmt == "" ->
             formatString [chr $ fromIntegral x] (ufmt { fmtChar = 's' })
     'c' -> perror "illegal char conversion"
@@ -731,9 +728,9 @@ fmtu b Nothing prec0 m0 i0 =
   where
     fmtu' :: Maybe Int -> Maybe Integer -> Integer -> Maybe String
     fmtu' prec (Just m) i | i < 0 =
-      fmtu' prec Nothing (-2 * m + i)
+      fmtu' prec Nothing (- (2 * m) + i)
     fmtu' (Just prec) _ i | i >= 0 =
-      fmap (integral_prec (Just prec)) $ fmtu' Nothing Nothing i
+      integral_prec (Just prec) <$> fmtu' Nothing Nothing i
     fmtu' Nothing _ i | i >= 0 =
       Just $ showIntAtBase b intToDigit i ""
     fmtu' _ _ _ = Nothing
