@@ -41,15 +41,15 @@ module Text.Read.Internal
   , readParen
   )
  where
-import Prelude()
+import qualified Prelude()
 
 import Control.Applicative
 import Control.Error
 import Control.Monad
 import Control.Monad.Fail
-import Data.Array
 import Data.Bool
 import Data.Char
+import Data.Coerce
 import Data.Double
 import Data.Either
 import Data.Eq
@@ -62,6 +62,7 @@ import Data.Integer
 import Data.Integral
 import Data.Ix
 import Data.List
+import Data.List.NonEmpty_Type
 import Data.Num
 import Data.Ord
 import Data.Ratio
@@ -69,11 +70,13 @@ import Data.Real
 import Data.RealFloat
 import Data.String
 import Data.Tuple
+import {-# SOURCE #-} Data.Typeable
 import Data.Word
-import Numeric.Natural
+import Foreign.Ptr
 import qualified Text.ParserCombinators.ReadP as P
 import Text.ParserCombinators.ReadP(ReadS, readP_to_S)
 import Text.ParserCombinators.ReadPrec
+import Text.Read.Lex (Lexeme(..)) -- needed for deriving
 import qualified Text.Read.Lex as L
 import Text.Show(appPrec)
 
@@ -225,7 +228,7 @@ class Read a where
   readsPrec    = readPrec_to_S readPrec
   readList     = readPrec_to_S (list readPrec) 0
   readPrec     = readS_to_Prec readsPrec
-  readListPrec = readS_to_Prec (\_ -> readList)
+  readListPrec = readS_to_Prec (const readList)
 
 readListDefault :: Read a => ReadS [a]
 -- ^ A possible replacement definition for the 'readList' method (GHC only).
@@ -339,7 +342,7 @@ list :: ReadPrec a -> ReadPrec [a]
 list readx =
   parens
   ( do expectP (L.Punc "[")
-       (listRest False +++ listNext)
+       listRest False +++ listNext
   )
  where
   listRest started =
@@ -480,8 +483,7 @@ instance Read Ordering where
   readList     = readListDefault
 
 -- | @since base-4.11.0.0
--- deriving instance Read a => Read (NonEmpty a)
--- XXX
+deriving instance Read a => Read (NonEmpty a)
 
 --------------------------------------------------------------
 -- Structure instances of Read: Maybe, List etc
@@ -539,7 +541,7 @@ instance (Read a, Read b) => Read (Either a b) where
            x <- step readPrec
            return (Right x))
     )
-    
+
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
@@ -552,17 +554,6 @@ instance Read a => Read [a] where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
--- | @since base-2.01
-instance  (Ix a, Read a, Read b) => Read (Array a b)  where
-    readPrec = parens $ prec appPrec $
-               do expectP (L.Ident "array")
-                  theBounds <- step readPrec
-                  vals   <- step readPrec
-                  return (array theBounds vals)
-
-    readListPrec = readListPrecDefault
-    readList     = readListDefault
-
 -- | @since base-2.01
 instance Read L.Lexeme where
   readPrec     = lexP
@@ -613,15 +604,15 @@ instance Read Word where
 
 -- | @since base-2.01
 instance Read Word8 where
-    readsPrec p s = [(fromIntegral (x::Int), r) | (x, r) <- readsPrec p s]
+    readsPrec p s = [(fromIntegral (x::Word), r) | (x, r) <- readsPrec p s]
 
 -- | @since base-2.01
 instance Read Word16 where
-    readsPrec p s = [(fromIntegral (x::Int), r) | (x, r) <- readsPrec p s]
+    readsPrec p s = [(fromIntegral (x::Word), r) | (x, r) <- readsPrec p s]
 
 -- | @since base-2.01
 instance Read Word32 where
-    readsPrec p s = [(fromInteger x, r) | (x, r) <- readsPrec p s]
+    readsPrec p s = [(fromIntegral (x::Word), r) | (x, r) <- readsPrec p s]
 
 -- | @since base-2.01
 instance Read Word64 where
@@ -633,13 +624,6 @@ instance Read Integer where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
--- | @since base-4.8.0.0
-instance Read Natural where
-  readsPrec d = map (\(n, s) -> (fromInteger n, s))
-                  . filter ((>= 0) . (\(x,_)->x)) . readsPrec d
-  readListPrec = readListPrecDefault
-  readList     = readListDefault
-
 -- | @since base-2.01
 instance Read Float where
   readPrec     = readNumber convertFrac
@@ -672,13 +656,6 @@ instance (Integral a, Ord a, Read a) => Read (Ratio a) where
 -- Tuple instances of Read, up to size 15
 ------------------------------------------------------------------------
 
--- | Reading a 'Void' value is always a parse error, considering
--- 'Void' as a data type with no constructors.
---
--- @since base-4.8.0.0
--- deriving instance Read Void
--- XXX
-
 -- | @since base-2.01
 instance Read () where
   readPrec =
@@ -692,8 +669,7 @@ instance Read () where
   readList     = readListDefault
 
 -- | @since base-4.15
--- deriving instance Read a => Read (Solo a)
--- XXX
+deriving instance Read a => Read (Solo a)
 
 -- | @since base-2.01
 instance (Read a, Read b) => Read (a,b) where
@@ -797,3 +773,21 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+instance Read Int8 where
+    readsPrec p s = [(fromIntegral (x::Int), r) | (x, r) <- readsPrec p s]
+
+instance Read Int16 where
+    readsPrec p s = [(fromIntegral (x::Int), r) | (x, r) <- readsPrec p s]
+
+instance Read Int32 where
+    readsPrec p s = [(fromIntegral (x::Int), r) | (x, r) <- readsPrec p s]
+
+instance Read Int64 where
+    readsPrec p s = [(fromInteger x, r) | (x, r) <- readsPrec p s]
+
+--------------------------------------------------------------
+-- Foreign.Ptr instances of Read
+--------------------------------------------------------------
+
+deriving newtype instance Read IntPtr
+deriving newtype instance Read WordPtr
