@@ -1,11 +1,12 @@
 -- Copyright 2023,2024 Lennart Augustsson
 -- See LICENSE file for full license.
 module Text.Show(module Text.Show) where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
 import Primitives
 import Data.Bool_Type
 import Data.Char_Type
 import Data.List_Type
+import {-# SOURCE #-} Data.Typeable
 
 type ShowS = String -> String
 
@@ -15,7 +16,7 @@ class Show a where
   showList  :: [a] -> ShowS
 
   showsPrec _ x s = show x ++ s
-  show x          = showsPrec 0 x ""
+  show x          = shows x ""
   showList        = showListWith shows
 
 shows :: forall a . Show a => a -> ShowS
