@@ -1,4 +1,4 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module PrimTable() where
-import Prelude()              -- do not import Prelude
+import qualified Prelude()              -- do not import Prelude
