@@ -3,11 +3,11 @@ module Paths_MicroHs(
   version,
   getDataDir,
   ) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
 import Data.Version
 
 getDataDir :: IO FilePath
 getDataDir = return "."
 
 version :: Version
-version = makeVersion [0,10,3,0]
+version = makeVersion [0,15,0,0]
