@@ -2,6 +2,9 @@
 -- See LICENSE file for full license.
 module MicroHs.Compile(
   compileCacheTop,
+  compileModuleP,
+  compileToCombinators,
+  compileInteractive,
   compileMany,
   maybeSaveCache,
   getCached,
@@ -11,14 +14,19 @@ module MicroHs.Compile(
   packageDir, packageSuffix, packageTxtSuffix,
   mhsVersion,
   getMhsDir,
+  openFilePath,
+  loadPkg,
+  addPreludeImport,
   ) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
+import Control.Exception
 import Data.Char
 import Data.List
 import Data.Maybe
 import Data.Version
 import System.Directory
 import System.Environment
+import System.FilePath
 import System.IO
 import System.IO.MD5
 import System.IO.Serialize
@@ -38,9 +46,8 @@ import MicroHs.Package
 import MicroHs.Parse
 import MicroHs.StateIO
 import MicroHs.SymTab
+import MicroHs.TCMonad(TCState)
 import MicroHs.TypeCheck
-import Compat
-import MicroHs.Instances() -- for ghc
 import Paths_MicroHs(version, getDataDir)
 
 mhsVersion :: String
@@ -51,6 +58,8 @@ mhsCacheName = ".mhscache"
 
 type Time = Int
 
+type CM a = StateIO Cache a
+
 -----------------
 
 -- Compile the module with the given name, starting with the given cache.
@@ -58,12 +67,15 @@ type Time = Int
 compileCacheTop :: Flags -> IdentModule -> Cache -> IO ((IdentModule, [(Ident, Exp)]), Symbols, Cache)
 compileCacheTop flags mn ch = do
   res@((_, ds), _, _) <- compile flags mn ch
-  when (verbosityGT flags 4) $
+  dumpIf flags Dcombinator $
     putStrLn $ "combinators:\n" ++ showLDefs ds
   return res
 
 compileMany :: Flags -> [IdentModule] -> Cache -> IO Cache
-compileMany flags mns ach = snd <$> runStateIO (mapM_ (compileModuleCached flags ImpNormal) mns) ach
+compileMany flags mns ach =
+  execStateIO (do mapM_ (loadPkg flags) (preload flags)
+                  mapM_ (compileModuleCached flags ImpNormal) mns)
+              ach
 
 getCached :: Flags -> IO Cache
 getCached flags | not (readCache flags) = return emptyCache
@@ -82,38 +94,47 @@ maybeSaveCache flags cash =
   when (writeCache flags) $ do
     when (verbosityGT flags 0) $
       putStrLn $ "Saving cache " ++ show mhsCacheName
-    -- This causes all kinds of chaos, probably because there
-    -- will be equality tests of unevaluated thunks.
-    -- () <- seq (rnfNoErr cash) (return ())
     saveCache mhsCacheName cash
 
+-- Load the real modules for imported boot modules.
+-- Doing so can result in more boot imports, so we recurse.
+loadBoots :: Flags -> CM ()
+loadBoots flags = do
+  bs <- gets getBoots
+  case bs of
+    [] -> return ()
+    bmn:_ -> do
+      when (verbosityGT flags 0) $
+        liftIO $ putStrLn $ "compiling used boot module " ++ showIdent bmn
+      _ <- compileModuleCached flags ImpNormal bmn
+      loadBoots flags
+
 compile :: Flags -> IdentModule -> Cache -> IO ((IdentModule, [LDef]), Symbols, Cache)
 compile flags nm ach = do
   let comp = do
---XXX        modify $ addBoot $ mkIdent "Control.Exception.Internal"      -- the compiler generates references to this module
-        r <- compileModuleCached flags ImpNormal nm
-        let loadBoots = do
-              bs <- gets getBoots
-              case bs of
-                [] -> return ()
-                bmn:_ -> do
-                  when (verbosityGT flags 0) $
-                    liftIO $ putStrLn $ "compiling used boot module " ++ showIdent bmn
-                  _ <- compileModuleCached flags ImpNormal bmn
-                  loadBoots
-        loadBoots
+        mapM_ (loadPkg flags) (preload flags)
+        res <- compileModuleCached flags ImpNormal nm
+        loadBoots flags
         loadDependencies flags
-        return r
+        return res
   ((cm, syms, t), ch) <- runStateIO comp ach
   when (verbosityGT flags 0) $
     putStrLn $ "total import time     " ++ padLeft 6 (show t) ++ "ms"
   return ((tModuleName cm, concatMap tBindingsOf $ cachedModules ch), syms, ch)
 
+-- Compile a module for the interactive system
+compileInteractive :: Flags -> EModule -> CM (TModule [LDef], Symbols, TCState)
+compileInteractive flags mdl = do
+  ((dmdl, syms, _, _, _), tcstate) <- compileModuleP flags ImpNormal mdl
+  loadBoots flags
+  loadDependencies flags
+  return (dmdl, syms, tcstate)
+
 -- Compile a module with the given name.
 -- If the module has already been compiled, return the cached result.
 -- If the module has not been compiled, first try to find a source file.
 -- If there is no source file, try loading a package.
-compileModuleCached :: Flags -> ImpType -> IdentModule -> StateIO Cache (TModule [LDef], Symbols, Time)
+compileModuleCached :: Flags -> ImpType -> IdentModule -> CM (TModule [LDef], Symbols, Time)
 compileModuleCached flags impt mn = do
   cash <- get
   case lookupCache mn cash of
@@ -124,11 +145,23 @@ compileModuleCached flags impt mn = do
           when (verbosityGT flags 1) $ do
             ms <- gets getWorking
             putStrLnInd $ "[from " ++ head (map showIdent ms ++ ["-"]) ++ "]"
-            putStrLnInd $ "importing " ++ showIdent mn
-          mres <- liftIO (readModulePath flags ".hs" mn)
+            putStrInd $ "importing " ++ showIdent mn
+          mres <- liftIO (readModulePath flags "hs" mn)
           case mres of
-            Nothing -> findPkgModule flags mn
+            Nothing -> do
+              (fn, res) <- findPkgModule flags mn
+              liftIO $ when (verbosityGT flags 1) $ do
+                when (verbosityGT flags 2) $
+                  putStrLn $ " (" ++ show fn ++ ")"
+                putStrLn ""
+              return res
             Just (pathfn, file) -> do
+              liftIO $ when (verbosityGT flags 1) $ do
+                when (verbosityGT flags 2) $
+                  putStrLn $ " (" ++ show pathfn ++ ")"
+                putStrLn ""
+              dumpIf flags Dpreproc $
+                liftIO $ putStrLn $ "preprocessed:\n" ++ file
               modify $ addWorking mn
               compileModule flags ImpNormal mn pathfn file
     Just tm -> do
@@ -136,87 +169,105 @@ compileModuleCached flags impt mn = do
         putStrLnInd $ "importing cached " ++ showIdent mn
       return (tm, noSymbols, 0)
 
-putStrLnInd :: String -> StateIO Cache ()
+putStrLnInd :: String -> CM ()
 putStrLnInd msg = do
   ms <- gets getWorking
   liftIO $ putStrLn $ map (const ' ') ms ++ msg
 
+putStrInd :: String -> CM ()
+putStrInd msg = do
+  ms <- gets getWorking
+  liftIO $ putStr $ map (const ' ') ms ++ msg
+
 noSymbols :: Symbols
 noSymbols = (stEmpty, stEmpty)
 
-compileBootModule :: Flags -> IdentModule -> StateIO Cache (TModule [LDef], Symbols, Time)
+compileBootModule :: Flags -> IdentModule -> CM (TModule [LDef], Symbols, Time)
 compileBootModule flags mn = do
   when (verbosityGT flags 0) $
     putStrLnInd $ "importing boot " ++ showIdent mn
   mres <- liftIO (readModulePath flags ".hs-boot" mn)
   case mres of
-    Nothing -> error $ "boot module not found: " ++ showIdent mn
+    Nothing -> mhsError $ "boot module not found: " ++ showIdent mn
     Just (pathfn, file) -> do
+      dumpIf flags Dpreproc $
+        liftIO $ putStrLn $ "preprocessed:\n" ++ file
       modify $ addBoot mn
       compileModule flags ImpBoot mn pathfn file
 
-compileModule :: Flags -> ImpType -> IdentModule -> FilePath -> String -> StateIO Cache (TModule [LDef], Symbols, Time)
+compileModule :: Flags -> ImpType -> IdentModule -> FilePath -> String -> CM (TModule [LDef], Symbols, Time)
 compileModule flags impt mn pathfn file = do
   t1 <- liftIO getTimeMilli
   mchksum <- liftIO (md5File pathfn)  -- XXX there is a small gap between reading and computing the checksum.
   let chksum :: MD5CheckSum
       chksum = fromMaybe undefined mchksum
-  let pmdl = parseDie pTop pathfn file
-      mdl@(EModule mnn _ defs) = addPreludeImport pmdl
   when (verbosityGT flags 4) $
     liftIO $ putStrLn $ "parsing: " ++ pathfn
-  when (verbosityGT flags 4) $
+  let pmdl@(EModule mnn _ _) = parseDie pTop pathfn file
+  t2 <- liftIO (seq pmdl getTimeMilli)
+  dumpIf flags Dparse $
     liftIO $ putStrLn $ "parsed:\n" ++ show pmdl
-  
-  -- liftIO $ putStrLn $ showEModule mdl
-  -- liftIO $ putStrLn $ showEDefs defs
-  -- TODO: skip test when mn is a file name
   when (isNothing (getFileName mn) && mn /= mnn) $
-    error $ "module name does not agree with file name: " ++ showIdent mn ++ " " ++ showIdent mnn
-  let
-    specs = [ s | Import s <- defs ]
-    imported = [ (boot, m) | ImportSpec boot _ m _ _ <- specs ]
-  t2 <- liftIO getTimeMilli
-  (impMdls, _, tImps) <- fmap unzip3 $ mapM (uncurry $ compileModuleCached flags) imported
+    mhsError $ "module name does not agree with file name: " ++ showIdent mn ++ " " ++ showIdent mnn
+  ((dmdl, syms, imported, tTCDesug, tImp), _) <- compileModuleP flags impt (addPreludeImport pmdl)
 
-  t3 <- liftIO getTimeMilli
-  glob <- gets getCacheTables
-  let
-    (tmdl, glob', syms) = typeCheck glob impt (zip specs impMdls) mdl
-  modify $ setCacheTables glob'
-  when (verbosityGT flags 3) $
-    liftIO $ putStrLn $ "type checked:\n" ++ showTModule showEDefs tmdl ++ "-----\n"
-  let
-    dmdl = desugar flags tmdl
-  () <- return $ rnfErr $ tBindingsOf dmdl
   t4 <- liftIO getTimeMilli
-
-  let
-    cmdl = setBindings dmdl [ (i, compileOpt e) | (i, e) <- tBindingsOf dmdl ]
-  () <- return $ rnfErr $ tBindingsOf cmdl  -- This makes execution slower, but speeds up GC
---  () <- return $ rnfErr syms same for this, but worse total time
+  cmdl <- liftIO $ evaluate $ compileToCombinators dmdl
   t5 <- liftIO getTimeMilli
 
   let tParse = t2 - t1
-      tTCDesug = t4 - t3
       tAbstract = t5 - t4
       tThis = tParse + tTCDesug + tAbstract
-      tImp = sum tImps
 
-  when (verbosityGT flags 4) $
-    (liftIO $ putStrLn $ "desugared:\n" ++ showTModule showLDefs dmdl)
+  dumpIf flags Ddesugar $
+    liftIO $ putStrLn $ "desugared:\n" ++ showTModule showLDefs dmdl
   when (verbosityGT flags 0) $
-    putStrLnInd $ "importing done " ++ showIdent mn ++ ", " ++ show tThis ++
-            "ms (" ++ show tParse ++ " + " ++ show tTCDesug ++ " + " ++ show tAbstract ++ ")"
+    putStrLnInd $ "importing done " ++ showIdent mn ++ ", " ++ show tThis ++ "ms " ++
+                  " (" ++ show tParse ++ " + " ++ show tTCDesug ++ " + " ++ show tAbstract ++ ")"
   when (loading flags && mn /= mkIdent "Interactive" && not (verbosityGT flags 0)) $
-    liftIO $ putStrLn $ "loaded " ++ showIdent mn
+    liftIO $ putStrLn $ "loaded " ++ showIdent mn ++ " (" ++ pathfn ++ ")"
 
   case impt of
     ImpNormal -> modify $ workToDone (cmdl, map snd imported, chksum)
     ImpBoot   -> return ()
-
   return (cmdl, syms, tThis + tImp)
 
+compileModuleP :: Flags -> ImpType -> EModule -> CM ((TModule [LDef], Symbols, [(ImpType, IdentModule)], Time, Time), TCState)
+compileModuleP flags impt mdl@(EModule _ _ defs) = do
+  -- liftIO $ putStrLn $ showEModule mdl
+  -- liftIO $ putStrLn $ showEDefs defs
+  let
+    specs = [ s | Import s <- defs ]
+    imported = [ (boot, m) | ImportSpec boot _ m _ _ <- specs ]
+  (impMdls, _, tImps) <- unzip3 <$> mapM (uncurry $ compileModuleCached flags) imported
+
+  t3 <- liftIO getTimeMilli
+  glob <- gets getCacheTables
+  let
+    (tmdl, glob', syms, tcstate) = typeCheck flags glob impt (zip specs impMdls) mdl
+  modify $ setCacheTables glob'
+  dumpIf flags Dtypecheck $
+    liftIO $ putStrLn $ "type checked:\n" ++ showTModule showEDefs tmdl ++ "-----\n"
+  let
+    dmdl = desugar flags tmdl
+  t4 <- liftIO getTimeMilli
+
+  let tThis = t4 - t3
+      tImp = sum tImps
+
+  return ((dmdl, syms, imported, tThis, tImp), tcstate)
+
+compileToCombinators :: TModule [LDef] -> TModule [LDef]
+compileToCombinators dmdl = do
+  let cmdl = setBindings dmdl [ (i, compileOpt e) | (i, e) <- tBindingsOf dmdl ]
+  seq (rnf cmdl) cmdl  -- This makes execution slower, but speeds up GC
+
+-- Add implicit imports:
+--   import Prelude
+--   import Mhs.Builting as B@
+-- No implicitr imports are added when
+--   import qualified Prelude()
+-- is present.
 addPreludeImport :: EModule -> EModule
 addPreludeImport (EModule mn es ds) =
   EModule mn es ds'
@@ -225,14 +276,12 @@ addPreludeImport (EModule mn es ds) =
         isImportPrelude (Import (ImportSpec _ _ i _ _)) = i == idPrelude
         isImportPrelude _ = False
         idPrelude = mkIdent "Prelude"
-        idBuiltin = mkIdent "Mhs.Builtin"
-        idB = mkIdent builtinMdl
-        iblt = Import $ ImportSpec ImpNormal True idBuiltin (Just idB) Nothing
+        iblt = Import $ ImportSpec ImpNormal True builtinMdlQ (Just builtinMdl) Nothing
         ps' =
           case ps of
             [] -> [Import $ ImportSpec ImpNormal False idPrelude Nothing Nothing,      -- no Prelude imports, so add 'import Prelude'
-                   iblt]                                                               -- and 'import Mhs.Builtin as @B'
-            [Import (ImportSpec ImpNormal False _ Nothing (Just (False, [])))] -> []   -- exactly 'import Prelude()', so import nothing
+                   iblt]                                                               -- and 'import Mhs.Builtin as B@'
+            [Import (ImportSpec ImpNormal True _ Nothing (Just (False, [])))] -> []    -- exactly 'import qualified Prelude()', so import nothing
             _ -> iblt : ps                                                             -- keep the given Prelude imports, add Builtin
 
 -------------------------------------------
@@ -242,7 +291,7 @@ validateCache flags acash = execStateIO (mapM_ (validate . fst) fdeps) acash
   where
     fdeps = getImportDeps acash                           -- forwards dependencies
     deps = invertGraph fdeps                              -- backwards dependencies
-    invalidate :: IdentModule -> StateIO Cache ()
+    invalidate :: IdentModule -> CM ()
     invalidate mn = do
       b <- gets $ isJust . lookupCache mn
       when b $ do
@@ -251,7 +300,7 @@ validateCache flags acash = execStateIO (mapM_ (validate . fst) fdeps) acash
           liftIO $ putStrLn $ "invalidate cached " ++ show mn
         modify (deleteFromCache mn)
         mapM_ invalidate $ fromMaybe [] $ M.lookup mn deps
-    validate :: IdentModule -> StateIO Cache ()
+    validate :: IdentModule -> CM ()
     validate mn = do
       cash <- get
       case lookupCacheChksum mn cash of
@@ -281,35 +330,47 @@ invertGraph = foldr ins M.empty
 
 -- Is the module name actually a file name?
 getFileName :: IdentModule -> Maybe String
-getFileName m | ".hs" `isSuffixOf` s = Just s
+getFileName m | ".hs" `isSuffixOf` s || ".lhs" `isSuffixOf` s ||
+                not (validModuleName s) = Just s
               | otherwise = Nothing
   where s = unIdent m
 
-readModulePath :: Flags -> String -> IdentModule -> IO (Maybe (FilePath, String))
-readModulePath flags suf mn | Just fn <- getFileName mn = do
-  mh <- openFileM fn ReadMode
-  case mh of
-    Nothing -> errorMessage (getSLoc mn) $ "File not found: " ++ show fn
-    Just h -> readRest fn h
+-- A hack to allow files that don't end in .hs
+validModuleName :: String -> Bool
+validModuleName (uc:s) = isUpper uc && all (\ c -> isAlphaNum c || c == '.' || c == '_' || c == '\'') s
+validModuleName _ = False
 
-                            | otherwise = do
-  mh <- findModulePath flags suf mn
-  case mh of
-    Nothing -> do
-      mhc <- findModulePath flags (suf ++ "c") mn  -- look for hsc file
-      case mhc of
-        Nothing -> return Nothing
-        Just (_fn, _h) -> undefined
-    Just (fn, h) -> readRest fn h
-  where readRest fn h = do
+readModulePath :: Flags -> String -> IdentModule -> IO (Maybe (FilePath, String))
+readModulePath flags suf mn = do
+  case getFileName mn of
+    Just fn -> do
+      mh <- openFileM fn ReadMode
+      case mh of
+        Nothing -> errorMessage (getSLoc mn) $ "File not found: " ++ show fn
+        Just h  -> readRest fn =<< hGetContents h
+    _ -> do
+      mh <- findModulePath flags suf mn
+      case mh of
+        Nothing -> do
+          mhc <- findModulePath flags (suf ++ "c") mn  -- look for hsc file
+          case mhc of
+            Nothing -> do
+              mhl <- findModulePath flags ("l" ++ suf) mn  -- look for lhs file
+              case mhl of
+                Nothing -> do
+                  return Nothing
+                Just (fn, h) -> readRest fn . unlit fn =<< hGetContents h
+            Just (fn, _h) -> readRest fn =<< runHsc2hs flags fn
+        Just (fn, h) -> readRest fn =<< hGetContents h
+  where readRest :: FilePath -> String -> IO (Maybe (FilePath, String))
+        readRest fn file = do
           hasCPP <- hasLangCPP fn
-          file <-
+          file' <-
             if hasCPP || doCPP flags then do
-              hClose h
-              runCPPTmp flags fn
+              runCPPString flags fn file
             else
-              hGetContents h
-          return (Just (fn, file))
+              return file
+          return (Just (fn, file'))
 
 -- Check if the file contains {-# LANGUAGE ... CPP ... #-}
 -- XXX This is pretty hacky and not really correct.
@@ -318,6 +379,7 @@ hasLangCPP fn = do
   let scanFor _ [] = False
       scanFor s ('{':'-':'#':cs) = scanFor' s cs
       scanFor _ ('m':'o':'d':'u':'l':'e':_) = False
+      scanFor s ('-':'-':cs) = scanFor s (dropWhile (/= '\n') cs)
       scanFor s (_:cs) = scanFor s cs
       scanFor' _ [] = False
       scanFor' s ('#':'-':'}':cs) = scanFor s cs
@@ -326,12 +388,12 @@ hasLangCPP fn = do
   scanFor "cpp" . map toLower <$> readFile fn
 
 moduleToFile :: IdentModule -> FilePath
-moduleToFile mn = map (\ c -> if c == '.' then '/' else c) (unIdent mn)
+moduleToFile mn = map (\ c -> if c == '.' then pathSeparator else c) (unIdent mn)
 
 findModulePath :: Flags -> String -> IdentModule -> IO (Maybe (FilePath, Handle))
 findModulePath flags suf mn = do
   let
-    fn = moduleToFile mn ++ suf
+    fn = moduleToFile mn <.> suf
   openFilePath (paths flags) fn
 
 openFilePath :: [FilePath] -> FilePath -> IO (Maybe (FilePath, Handle))
@@ -340,19 +402,23 @@ openFilePath adirs fileName =
     [] -> return Nothing
     dir:dirs -> do
       let
-        path = dir ++ "/" ++ fileName
+        path = dir </> fileName
       mh <- openFileM path ReadMode
       case mh of
         Nothing -> openFilePath dirs fileName -- If opening failed, try the next directory
         Just hdl -> return (Just (path, hdl))
 
-runCPPTmp :: Flags -> FilePath -> IO String
-runCPPTmp flags infile = do
-  (fn, h) <- openTmpFile "mhscpp.hs"
-  runCPP flags infile fn
-  file <- hGetContents h
-  removeFile fn
-  return file
+runCPPString :: Flags -> FilePath -> String -> IO String
+runCPPString flags fn ifile = do
+  (fni, hi) <- openTmpFile "mhsin.hs"
+  hClose hi
+  writeFile fni $ "#line 1 \"" ++ fn ++ "\"\n" ++ ifile
+  (fno, ho) <- openTmpFile "mhsout.hs"
+  runCPP flags fni fno
+  ofile <- hGetContents ho
+  removeFile fni
+  removeFile fno
+  return ofile
 
 mhsDefines :: [String]
 mhsDefines =
@@ -364,14 +430,33 @@ runCPP flags infile outfile = do
   mcpphs <- lookupEnv "MHSCPPHS"
   datadir <- getMhsDir
   let cpphs = fromMaybe "cpphs" mcpphs
-      mhsIncludes = ["-I" ++ datadir ++ "/src/runtime"]
+      mhsIncludes = ["-I" ++ datadir </> "src/runtime"]
       args = mhsDefines ++ mhsIncludes ++ map quote (cppArgs flags)
       cmd = cpphs ++ " --strip " ++ unwords args ++ " " ++ infile ++ " -O" ++ outfile
-      quote s = "'" ++ s ++ "'"
   when (verbosityGT flags 1) $
     putStrLn $ "Run cpphs: " ++ show cmd
   callCommand cmd
 
+quote :: String -> String
+quote s = "'" ++ s ++ "'"
+
+runHsc2hs :: Flags -> FilePath -> IO String
+runHsc2hs flags fni = do
+  (fno, ho) <- openTmpFile "mhshsc2hs.hs"
+  mhsc2hs <- lookupEnv "MHSHSC2HS"
+  datadir <- getMhsDir
+  let hsc2hs = fromMaybe "hsc2hs" mhsc2hs
+      mhsIncludes = ["-I" ++ datadir </> "src/runtime"
+                    ,"-I" ++ datadir </> "src/runtime/unix"]
+      args = mhsDefines ++ mhsIncludes ++ map quote (cppArgs flags)
+      cmd = unwords $ [hsc2hs, "-o", fno] ++ args ++ [fni]
+  when (verbosityGT flags 1) $
+    putStrLn $ "Run hsc2hs: " ++ show cmd
+  callCommand cmd
+  ofile <- hGetContents ho
+  removeFile fno
+  return ofile  
+
 packageDir :: String
 packageDir = "packages"
 packageSuffix :: String
@@ -380,65 +465,81 @@ packageTxtSuffix :: String
 packageTxtSuffix = ".txt"
 
 -- Find the module mn in the package path, and return it's contents.
-findPkgModule :: Flags -> IdentModule -> StateIO Cache (TModule [LDef], Symbols, Time)
+findPkgModule :: Flags -> IdentModule -> CM (FilePath, (TModule [LDef], Symbols, Time))
 findPkgModule flags mn = do
   t0 <- liftIO getTimeMilli
-  let fn = moduleToFile mn ++ packageTxtSuffix
+  let fn = moduleToFile mn <.> packageTxtSuffix
   mres <- liftIO $ openFilePath (pkgPath flags) fn
   case mres of
     Just (pfn, hdl) -> do
       -- liftIO $ putStrLn $ "findPkgModule " ++ pfn
       pkg <- liftIO $ hGetContents hdl  -- this closes the handle
       let dir = take (length pfn - length fn) pfn  -- directory where the file was found
-      loadPkg flags (dir ++ packageDir ++ "/" ++ pkg)
+      loadPkg flags (dir ++ packageDir </> pkg)
       cash <- get
       case lookupCache mn cash of
-        Nothing -> error $ "package does not contain module " ++ pkg ++ " " ++ showIdent mn
+        Nothing -> mhsError $ "package does not contain module " ++ pkg ++ " " ++ showIdent mn
         Just t -> do
           t1 <- liftIO getTimeMilli
-          return (t, noSymbols, t1 - t0)
+          return (pfn, (t, noSymbols, t1 - t0))
     Nothing ->
       errorMessage (getSLoc mn) $
         "Module not found: " ++ show mn ++
         "\nsearch path=" ++ show (paths flags) ++
         "\npackage path=" ++ show (pkgPath flags)
 
-loadPkg :: Flags -> FilePath -> StateIO Cache ()
+loadPkg :: Flags -> FilePath -> CM ()
 loadPkg flags fn = do
   when (loading flags || verbosityGT flags 0) $
     liftIO $ putStrLn $ "Loading package " ++ fn
   pkg <- liftIO $ readSerialized fn
   when (pkgCompiler pkg /= mhsVersion) $
-    error $ "Package compile version mismatch: package=" ++ pkgCompiler pkg ++ ", compiler=" ++ mhsVersion
+    mhsError $ "Package compile version mismatch: file=" ++ fn ++ ", package=" ++ pkgCompiler pkg ++ ", compiler=" ++ mhsVersion
   modify $ addPackage fn pkg
 
 -- XXX add function to find&load package from package name
 
 -- Load all packages that we depend on, but that are not already loaded.
-loadDependencies :: Flags -> StateIO Cache ()
+loadDependencies :: Flags -> CM ()
 loadDependencies flags = do
   loadedPkgs <- gets getPkgs
   let deps = concatMap pkgDepends loadedPkgs
       loaded = map pkgName loadedPkgs
-      deps' = [ p | (p, _v) <- deps, p `notElem` loaded ]
+      deps' = [ pv | pv@(p, _v) <- deps, p `notElem` loaded ]
   if null deps' then
     return ()
    else do
     mapM_ (loadDeps flags) deps'
     loadDependencies flags  -- loadDeps can add new dependencies
 
-loadDeps :: Flags -> IdentPackage -> StateIO Cache ()
-loadDeps flags pid = do
-  mres <- liftIO $ openFilePath (pkgPath flags) (packageDir ++ "/" ++ unIdent pid ++ packageSuffix)
+loadDeps :: Flags -> (IdentPackage, Version) -> CM ()
+loadDeps flags (pid, pver) = do
+  mres <- liftIO $ openFilePath (pkgPath flags) (packageDir </> unIdent pid ++ "-" ++ showVersion pver <.> packageSuffix)
   case mres of
-    Nothing -> error $ "Cannot find package " ++ showIdent pid
+    Nothing -> mhsError $ "Cannot find package " ++ showIdent pid
     Just (pfn, hdl) -> do
       liftIO $ hClose hdl
       loadPkg flags pfn
 
 getMhsDir :: IO FilePath
-getMhsDir = do
-  md <- lookupEnv "MHSDIR"
-  case md of
-    Just d -> return d
-    Nothing -> getDataDir
+getMhsDir = maybe getDataDir return =<< lookupEnv "MHSDIR"
+
+-- Deal with literate Haskell
+unlit :: FilePath -> String -> String
+unlit fn = unlines . un 1 True . lines
+  where
+    un :: Int -> Bool -> [String] -> [String]
+    un _ _ [] = []
+    un n _ (l:ls) | all isSpace l = un (n+1) True ls
+    un n _ ("\\begin{code}":ls) = "" : code (n+1) ls
+    un n spc (l@('#':_) : ls) = l : un (n+1) spc ls
+    un n spc (('>':l):ls) | spc = (' ':l) : un (n+1) True ls
+                          | otherwise = err n "missing blank before >"
+    un n _ (_:ls) = un (n+1) False ls
+    -- We allow non-blank to follow directly after >
+    code :: Int -> [String] -> [String]
+    code n ("\\end{code}":ls) = "" : un (n+1) False ls
+    code n (l : ls) = l : code (n+1) ls
+    code n [] = err n "unlit: missing \\end{code}"
+    err :: Int -> String -> a
+    err n s = error $ "unlit: " ++ fn ++ ":" ++ show n ++ ": " ++ s
