@@ -6,7 +6,7 @@ module MicroHs.Desugar(
   LDef, showLDefs,
   encodeInteger,
   ) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
 import Data.Char
 import Data.Function
 import Data.Integer(_integerToIntList)
@@ -14,7 +14,6 @@ import Data.List
 import Data.Maybe
 import Data.Ratio
 import Debug.Trace
-import GHC.Stack
 
 import MicroHs.EncodeData
 import MicroHs.Expr
@@ -23,6 +22,7 @@ import MicroHs.Flags
 import MicroHs.Graph
 import MicroHs.Ident
 import MicroHs.List
+import MicroHs.Names
 import MicroHs.State as S
 import MicroHs.TypeCheck
 
@@ -30,46 +30,58 @@ type LDef = (Ident, Exp)
 
 desugar :: Flags -> TModule [EDef] -> TModule [LDef]
 desugar flags tm =
-  setBindings tm (map lazier $ checkDup $ concatMap (dsDef flags (tModuleName tm)) (tBindingsOf tm))
+  setBindings tm $ map lazier $ checkDup $ concat $ zipWith (dsDef flags (tModuleName tm)) [1..] (tBindingsOf tm)
 
-dsDef :: Flags -> IdentModule -> EDef -> [LDef]
-dsDef flags mn adef =
+dsDef :: Flags -> IdentModule -> Int -> EDef -> [LDef]
+dsDef flags mn ffiNo adef =
   case adef of
     Data _ cs _ ->
       let
         n = length cs
-        dsConstr i (Constr _ ctx c ets) =
+        dsConstr i (Constr _ ctx c _ ets) =
           let
             ss = (if null ctx then [] else [False]) ++
                  map fst (either id (map snd) ets)   -- strict flags
           in (qualIdent mn c, encConstr i n ss)
       in  zipWith dsConstr [0::Int ..] cs
-    Newtype _ (Constr _ _ c _) _ -> [ (qualIdent mn c, Lit (LPrim "I")) ]
-    Type _ _ -> []
+    Newtype _ (Constr _ _ c _ _) _ -> [ (qualIdent mn c, Lit (LPrim "I")) ]
     Fcn f eqns -> [(f, wrapTick (useTicks flags) f $ dsEqns (getSLoc f) eqns)]
-    Sign _ _ -> []
-    KindSign _ _ -> []
-    Import _ -> []
-    ForImp ie i t -> [(i, if isIO t then frgn else App perf frgn)]
-      where frgn = Lit $ LForImp (fromMaybe (unIdent (unQualIdent i)) ie) cty
-            cty = CType t
-            perf = Lit $ LPrim "IO.performIO"
-            isIO x | Just (_, r) <- getArrow x = isIO r
-            isIO (EApp (EVar io) _) = io == mkIdent "Primitives.IO"
-            isIO _ = False
-    Infix _ _ -> []
+    ForImp cc ie i t -> [(i, ccall t $ Lit $ mkForImp ffiNo cc ie i t)]
+    -- Foreign exports don't fit very well into the desugared syntax.
+    -- We represent
+    --   foreign export "foo" bar :: ty
+    -- with
+    --   foo = FE bar' ty'
+    -- where bar' is the desugared expression for bar, and ty' is the C type
+    -- (currently just a newtype of an EType).
+    ForExp _ (Just s) e t ->  [(mkIdentSLoc l s, mkForExp e' (CType t))]
+      where l = getSLoc e
+            e' = dsExpr e
     Class ctx (c, _) _ bs ->
       let f = mkIdent "$f"
           meths :: [Ident]
-          meths = [ qualIdent mn i | (BSign i _) <- bs ]
+          meths = [ qualIdent mn i | (Sign is _) <- bs, i <- is ]
           supers :: [Ident]
           supers = [ qualIdent mn $ mkSuperSel c i | i <- [1 .. length ctx] ]
           xs = [ mkIdent ("$x" ++ show j) | j <- [ 1 .. length ctx + length meths ] ]
       in  (qualIdent mn $ mkClassConstructor c, lams xs $ Lam f $ apps (Var f) (map Var xs)) :
           zipWith (\ i x -> (i, Lam f $ App (Var f) (lams xs $ Var x))) (supers ++ meths) xs
-    Instance _ _ -> []
-    Default _ -> []
-    Pattern _ _ -> []
+    _ -> []
+
+-- Code for a 'foreign import'.
+-- If the function is impure then wrap a performIO around the call.
+ccall :: EType -> Exp -> Exp
+ccall t f =
+  case getArrows t of
+    (_, EApp (EVar io) _) | io == identIO ->
+      f
+    (as, _) ->
+      -- pure function, need a performIO.
+      --   ^f  -->  \ x1 ... xn -> performIO (^f x1 ... xn)
+      let xs = map (\ i -> mkIdent $ '_' : show i) [1..length as]
+          perf = Lit $ LPrim "IO.performIO"
+          call = App perf $ apps f (map Var xs)
+      in  foldr Lam call xs
 
 wrapTick :: Bool -> Ident -> Exp -> Exp
 wrapTick False _ ee = ee
@@ -85,34 +97,33 @@ wrapTick True  i ee = wrap 0 ee
                        _ -> e
 -}
 
-oneAlt :: Expr -> EAlts
-oneAlt e = EAlts [([], e)] []
-
 dsBind :: Ident -> EBind -> [LDef]
 dsBind v abind =
   case abind of
-    BFcn f eqns -> [(f, dsEqns (getSLoc f) eqns)]
-    BPat p e ->
-      let
-        de = (v, dsExpr e)
-        ds = [ (i, dsExpr (ECase (EVar v) [(p, oneAlt $ EVar i)])) | i <- patVars p ]
-      in  de : ds
-    BSign _ _ -> []
-    BDfltSign _ _ -> []
+    Fcn f eqns -> [(f, dsEqns (getSLoc f) eqns)]
+    PatBind p e -> dsPatBind v p e
+    _ -> []
+
+-- Desugaring ~p in case introduces PatBind, so we need to get rid of it again.
+dsPatBind :: Ident -> EPat -> Expr -> [LDef]
+dsPatBind v p e =
+  let de = (v, dsExpr e)
+      ds = [ (i, dsExpr (ECase (EVar v) [(p, oneAlt $ EVar i)])) | i <- patVars p ]
+  in  de : ds
 
 dsEqns :: SLoc -> [Eqn] -> Exp
 dsEqns loc eqns =
   case eqns of
     Eqn aps _ : _ ->
       let
-        vs = allVarsBind $ BFcn (mkIdent "") eqns
+        vs = allVarsBind $ Fcn (mkIdent "") eqns
         xs = take (length aps) $ newVars "$q" vs
         mkArm (Eqn ps alts) =
           let ps' = map dsPat ps
-          in  (ps', dsAlts alts)
+          in  (ps', id, dsAlts alts)
         ex = dsCaseExp loc (vs ++ xs) (map Var xs) (map mkArm eqns)
       in foldr Lam ex xs
-    _ -> impossible
+    _ -> eMatchErr loc
 
 dsAlts :: EAlts -> (Exp -> Exp)
 dsAlts (EAlts alts bs) = dsBinds bs . dsAltsL alts
@@ -132,15 +143,18 @@ dsAlt dflt (SBind p e : ss) rhs = ECase e [(p, EAlts [(ss, rhs)] []), (EVar dumm
 dsAlt dflt (SThen (EVar i) : ss) rhs | isIdent "Data.Bool.otherwise" i = dsAlt dflt ss rhs
 dsAlt dflt (SThen e   : ss) rhs = EIf e (dsAlt dflt ss rhs) dflt
 dsAlt dflt (SLet bs   : ss) rhs = ELet bs (dsAlt dflt ss rhs)
+dsAlt _    (SRec _ : _) _ = impossible
 
 dsBinds :: [EBind] -> Exp -> Exp
 dsBinds [] ret = ret
-dsBinds ads@(BPat (ELazy False p) e : ds) ret =
+{-
+dsBinds ads@(PatBind (ELazy False p) e : ds) ret =
   -- Turn a strict let/where into a case.
   -- XXX This does no reordering of bindings.
   let rest = dsBinds ds ret
       used = allVarsExp ret ++ allVarsExpr (ELet ads (ETuple []))
   in  dsCaseExp (getSLoc p) used [dsExpr e] [([dsPat p], const rest)]
+-}
 dsBinds ads ret =
   let
     avs = concatMap allVarsBind ads
@@ -167,6 +181,8 @@ letE :: Ident -> Exp -> Exp -> Exp
 letE i e b = eLet i e b          -- do some minor optimizations
              --App (Lam i b) e
 
+-- Do a single recursive definition 'let i = e in b'
+-- by 'let i = Y (\i.e) in b'
 letRecE :: Ident -> Exp -> Exp -> Exp
 letRecE i e b = letE i (App (Lit (LPrim "Y")) (Lam i e)) b
 
@@ -185,7 +201,7 @@ letRecE i e b = letE i (App (Lit (LPrim "Y")) (Lam i e)) b
 --    in  body
 mutualRec :: Ident -> [LDef] -> Exp -> Exp
 mutualRec v ies body =
-  let !(is, es) = unzip ies
+  let (is, es) = unzip ies
       n = length is
       ev = Var v
       one m i = letE i (mkTupleSelE m n ev)
@@ -193,8 +209,10 @@ mutualRec v ies body =
   in  letRecE v (bnds $ mkTupleE es) $
       bnds body
 
+-- In case we are cross compiling for a 32 bit platform we don't want integers that are too big.
+-- So we use the 32 bit bounds on the int encoding.
 encodeInteger :: Integer -> Exp
-encodeInteger i | toInteger (minBound::Int) <= i && i < toInteger (maxBound::Int) =
+encodeInteger i | -0x80000000 <= i && i <= 0x7fffffff  =
 --  trace ("*** small integer " ++ show i) $
   App (Var (mkIdent "Data.Integer_Type._intToInteger")) (Lit (LInt (fromInteger i)))
                 | otherwise =
@@ -208,11 +226,17 @@ encodeRational r =
 dsExpr :: Expr -> Exp
 dsExpr aexpr =
   case aexpr of
+    -- Change calls to error&undefined to include location
+    EVar f | f == mkIdent "Control.Error.undefined" ->
+      dsExpr $ addLoc (mkIdentSLoc (getSLoc f) "Control.Error._undefinedLoc")
+    EVar f | f == mkIdent "Control.Error.error" ->
+      dsExpr $ addLoc (mkIdentSLoc (getSLoc f) "Control.Error._errorLoc")
+
     EVar i -> Var i
-    EApp (EApp (EVar app) (EListish (LCompr e stmts))) l | app == mkIdent "Data.List_Type.++" ->
+    EApp (EApp (EVar app) (EListish (LCompr e stmts))) l | app == iapp ->
       dsExpr $ dsCompr e stmts l
     EApp f a -> App (dsExpr f) (dsExpr a)
-    ELam qs -> dsEqns (getSLoc aexpr) qs
+    ELam l qs -> dsEqns l qs
     ELit l (LExn s) -> Var (mkIdentSLoc l s)
     ELit _ (LChar c) -> Lit (LInt (ord c))
     ELit _ (LInteger i) -> encodeInteger i
@@ -233,22 +257,27 @@ dsExpr aexpr =
             in foldr Lam body xs
           Nothing -> Var (conIdent c)
     _ -> impossibleShow aexpr
+  where addLoc i = EApp (EVar i) (ELit l (LStr (show l ++ ": "))) where l = getSLoc i
+        iapp = mkIdent "Data.List_Type.++"
 
 dsCompr :: Expr -> [EStmt] -> Expr -> Expr
-dsCompr e [] l = EApp (EApp consCon e) l
+dsCompr e [] l = EApp (EApp conCons e) l
 dsCompr e (SThen c : ss) l = EIf c (dsCompr e ss l) l
 dsCompr e (SLet ds : ss) l = ELet ds (dsCompr e ss l)
+-- Special case for the idiom [ ... | ..., p <- [x], ... ].  This is a little more efficient.
+dsCompr e (SBind p (EListish (LList [x])) : ss) l = ECase x [(p, oneAlt $ dsCompr e ss l), (EVar dummyIdent, oneAlt l)]
 dsCompr e xss@(SBind p g : ss) l = ELet [hdef] (EApp eh g)
   where
-    hdef = BFcn h [eqn1, eqn2, eqn3]
-    eqn1 = eEqn [nilCon] l
-    eqn2 = eEqn [EApp (EApp consCon p) vs] (dsCompr e ss (EApp eh vs))
-    eqn3 = eEqn [EApp (EApp consCon u) vs]               (EApp eh vs)
+    hdef = Fcn h [eqn1, eqn2, eqn3]
+    eqn1 = eEqn [conNil] l
+    eqn2 = eEqn [EApp (EApp conCons p) vs] (dsCompr e ss (EApp eh vs))
+    eqn3 = eEqn [EApp (EApp conCons u) vs]               (EApp eh vs)
     u = EVar dummyIdent
     h = head $ newVars "$h" allVs
     eh = EVar h
     vs = EVar $ head $ newVars "$vs" allVs
     allVs = allVarsExpr (EListish (LCompr (ETuple [e,l]) xss))  -- all used identifiers
+dsCompr _ (SRec _ : _) _ = impossible
 
 -- Use tuple encoding to make a tuple
 mkTupleE :: [Exp] -> Exp
@@ -269,34 +298,16 @@ dsPat apat =
     EVar _ -> apat
     ECon _ -> apat
     EApp f a -> EApp (dsPat f) (dsPat a)
-    EListish (LList ps) -> dsPat $ foldr (\ x xs -> EApp (EApp consCon x) xs) nilCon ps
+    EListish (LList ps) -> dsPat $ foldr (EApp . EApp conCons) conNil ps
     ETuple ps -> dsPat $ foldl EApp (tupleCon (getSLoc apat) (length ps)) ps
     EAt i p -> EAt i (dsPat p)
     ELit loc (LStr cs) | length cs < 2 -> dsPat (EListish (LList (map (ELit loc . LChar) cs)))
     ELit _ _ -> apat
     ENegApp _ -> apat
-    EViewPat _ _ -> apat
+    EViewPat e p -> EViewPat e (dsPat p)
     ELazy b pat -> ELazy b (dsPat pat)
     _ -> impossible
 
-iNil :: Ident
-iNil = mkIdent $ listPrefix ++ "[]"
-
-iCons :: Ident
-iCons = mkIdent $ listPrefix ++ ":"
-
-consCon :: EPat
-consCon = ECon $ ConData [(iNil, 0), (iCons, 2)] iCons []
-
-nilCon :: EPat
-nilCon = ECon $ ConData [(iNil, 0), (iCons, 2)] iNil []
-
-tupleCon :: SLoc -> Int -> EPat
-tupleCon loc n =
-  let
-    c = tupleConstr loc n
-  in ECon $ ConData [(c, n)] c []
-
 newVars :: String -> [Ident] -> [Ident]
 newVars s is = deleteAllsBy (==) [ mkIdent (s ++ show i) | i <- [1::Int ..] ] is
 
@@ -321,12 +332,13 @@ dsCase loc ae as =
     mkArm :: ECaseArm -> Arm
     mkArm (p, alts) =
       let p' = dsPat p
-      in  ([p'], dsAlts alts)
+      in  ([p'], id, dsAlts alts)
 
 type MState = [Ident]  -- supply of unused variables.
 
 type M a = State MState a
-type Arm = ([EPat], Exp -> Exp)  -- Patterns, and a function that expects the default (which might be ignored).
+type Arm = ([EPat], Exp -> Exp, Exp -> Exp)  -- Patterns, a substitution,
+                                             -- and a function that expects the default (which might be ignored).
 type Matrix = [Arm]
 
 newIdents :: Int -> M [Ident]
@@ -357,27 +369,38 @@ dsMatrixL :: HasCallStack =>
 dsMatrixL dflt is arms = dsMatrix dflt is (map dsLazy arms)
 
 dsLazy :: Arm -> Arm
-dsLazy (ps, rhs) =
+dsLazy (ps, sub, rhs) =
+  -- Accumulate lazy bindings and strict bindings
   let ((_, rbs, ris), ps') = mapAccumL lazy (1, [], []) ps
       lazy :: (Int, [EBind], [Exp]) -> EPat -> ((Int, [EBind], [Exp]), EPat)
       lazy s@(n, bs, is) ap =
         case ap of
-          ELazy False p'@(EVar i) -> ((n, bs, Var i : is), p')
-          ELazy False p'          -> lazy (n, bs, is) p'
+          ELazy False p'@(EVar i) | not (isDummyIdent i)        -- XXX !_ doesn't work
+                                  -> ((n, bs, Var i : is), p')
+          ELazy False p'          -> lazy (n, bs, is) p'        -- ignore ! on non-variables for now
           ELazy True  p'          -> ((n+1, b:bs, is), EVar v)
             where v = mkIdent ("~" ++ show n)
-                  b = BPat p' (EVar v)
+                  b = PatBind p' (EVar v)
           EVar _                  -> (s, ap)
           EViewPat e p            -> (s', EViewPat e p') where (s', p')  = lazy s p
           ECon _                  -> (s, ap)
           EApp p1 p2              -> (s'', EApp p1' p2') where (s', p1') = lazy s p1; (s'', p2') = lazy s' p2
           EAt i p                 -> (s', EAt i p')      where (s', p')  = lazy s p
           _                       -> impossible
-  in  (ps', \ d -> dsBinds (reverse rbs) $ foldr eSeq (rhs d) (reverse ris))
+  in  (ps', sub, \ d -> dsBinds (reverse rbs) $ foldr eSeq (rhs d) (reverse ris))
 
 eSeq :: Exp -> Exp -> Exp
 eSeq e1 e2 = App (App (Lit (LPrim "seq")) e1) e2
 
+-- XXX quadratic.  but only used for short lists
+groupEq :: forall a . (a -> a -> Bool) -> [a] -> [[a]]
+groupEq eq axs =
+  case axs of
+    [] -> []
+    x:xs ->
+      case partition (eq x) xs of
+        (es, ns) -> (x:es) : groupEq eq ns
+
 -- Desugar a pattern matrix.
 -- The input is a (usually identifier) vector e1, ..., en
 -- and patterns matrix p11, ..., p1n   -> e1
@@ -391,33 +414,35 @@ dsMatrix dflt _ [] = return dflt
 dsMatrix dflt []         aarms =
   -- We can have several arms if there are guards.
   -- Combine them in order.
-  return $ foldr (\ (_, rhs) -> rhs) dflt aarms
-dsMatrix dflt iis@(i:is) aarms@(aarm : aarms') =
+  return $ foldr (\ (_, sub, rhs) -> sub . rhs) dflt aarms
+dsMatrix dflt iis@(i:is) aarms@(aarm : _) =
   case leftMost aarm of
     EVar _ -> do
       -- Find all variables, substitute with i, and proceed
-      let !(vars, nvars) = span (isPVar . leftMost) aarms
+      let (vars, nvars) = span (isPVar . leftMost) aarms
           vars' = map (sub . unAt i) vars
-          sub (EVar x : ps, rhs) = (ps, substAlpha x i . rhs)
+          sub (EVar x : ps, sb, rhs) = (ps, substAlpha x i . sb, rhs)
           sub _ = impossible
       letBind (dsMatrix dflt iis nvars) $ \ drest ->
         dsMatrix drest is vars'
     -- Collect identical transformations, do the transformation and proceed.
     EViewPat e _ -> do
-      let (views, nviews) = span (isPView e . leftMost) aarms'
+      let e' = case unAt i aarm of (_, sub, _) -> sub (dsExpr e)
+      let (views, nviews) = span (isPView e') (map (unAt i) aarms)
       letBind (dsMatrix dflt iis nviews) $ \ drest ->
-        letBind (return $ App (dsExpr e) i) $ \ vi -> do
-        let views' = map (unview . unAt i) (aarm : views)
-            unview (EViewPat _ p:ps, rhs) = (p:ps, rhs)
+        letBind (return $ App e' i) $ \ vi -> do
+        let views' = map unview views
+            unview (EViewPat _ p:ps, sub, rhs) = (p:ps, sub, rhs)
             unview _ = impossible
         dsMatrix drest (vi:is) views'
+
     -- Collect all constructors, group identical ones.
     _ -> do             -- must be ECon/EApp
       let
         (cons, ncons) = span (isPCon . leftMost) aarms
       letBind (dsMatrix dflt iis ncons) $ \ drest -> do
         let
-          idOf (p:_, _) = pConOf p
+          idOf (p:_, _, _) = pConOf p
           idOf _ = impossible
           grps = groupEq (on (==) idOf) $ map (unAt i) cons
           oneGroup grp = do
@@ -425,14 +450,14 @@ dsMatrix dflt iis@(i:is) aarms@(aarm : aarms') =
               con = pConOf $ leftMost $ head grp
             xs <- newIdents (conArity con)
             let
-              one (p : ps, e) = (pArgs p ++ ps, e)
+              one (p : ps, sub, e) = (pArgs p ++ ps, sub, e)
               one _ = impossible
             cexp <- dsMatrix drest (map Var xs ++ is) (map one grp)
             return (SPat con xs, cexp)
         narms <- mapM oneGroup grps
         return $ mkCase i narms drest
   where
-    leftMost (p:_, _) = skipEAt p  -- pattern in first column
+    leftMost (p:_, _, _) = skipEAt p  -- pattern in first column
     leftMost _ = impossible
     skipEAt (EAt _ p) = skipEAt p
     skipEAt p = p
@@ -441,10 +466,13 @@ dsMatrix dflt iis@(i:is) aarms@(aarm : aarms') =
     isPCon _ = True
     isPVar (EVar _) = True
     isPVar _ = False
-    isPView e (EViewPat e' _) = eqExpr e e'
+    isPView :: Exp -> Arm -> Bool
+    isPView e (EViewPat ee _:_, sub, _) = e == sub (dsExpr ee)
     isPView _ _ = False
-    unAt ii (EAt x p : ps, rhs) = unAt i (p:ps, substAlpha x ii . rhs)
-    unAt _ arm = arm
+
+unAt :: Exp{-Ident-} -> Arm -> Arm
+unAt ii (EAt x p : ps, sub, rhs) = unAt ii (p:ps, substAlpha x ii . sub, rhs)
+unAt _ arm = arm
 
 mkCase :: Exp -> [(SPat, Exp)] -> Exp -> Exp
 mkCase var pes dflt =
@@ -476,10 +504,12 @@ cheap :: Exp -> Bool
 cheap ae =
   case ae of
     Var _ -> True
-    Lit _ -> True
+    Lit (LInt _) -> True
+--    Lit _ -> True  -- inlining all literals can reduce sharing
     _ -> False
 
 eLet :: Ident -> Exp -> Exp -> Exp
+eLet i e b | cheap e = substExp i e b    -- always inline variables and literals
 eLet i e b =
   if i == dummyIdent then
     b
@@ -518,21 +548,12 @@ pArgs apat =
     ELit _ _ -> []
     _ -> impossible
 
--- XXX quadratic
-groupEq :: forall a . (a -> a -> Bool) -> [a] -> [[a]]
-groupEq eq axs =
-  case axs of
-    [] -> []
-    x:xs ->
-      case partition (eq x) xs of
-        (es, ns) -> (x:es) : groupEq eq ns
-
-getDups :: forall a . (a -> a -> Bool) -> [a] -> [[a]]
-getDups eq = filter ((> 1) . length) . groupEq eq
+getDups :: (Ord a) => [a] -> [[a]]
+getDups = filter ((> 1) . length) . groupSort
 
 checkDup :: [LDef] -> [LDef]
 checkDup ds =
-  case getDups (==) (filter (/= dummyIdent) $ map fst ds) of
+  case getDups $ filter (/= dummyIdent) $ map fst ds of
     [] -> ds
     (i1:_i2:_) : _ ->
       errorMessage (getSLoc i1) $ "duplicate definition " ++ showIdent i1
@@ -548,7 +569,7 @@ checkDup ds =
 -- thus avoiding the extra argument passing.
 -- This gives a small speedup with overloading.
 lazier :: LDef -> LDef
-lazier def@(fcn, l@(Lam _ (Lam _ _))) =
+lazier def@(fcn, l@(Lam _ _)) =
   let fcn' = addIdentSuffix fcn "@"
       vfcn' = Var fcn'
       args :: Exp -> (Exp, [Ident])
@@ -565,6 +586,7 @@ lazier def@(fcn, l@(Lam _ (Lam _ _))) =
       minn a 0 = a
       minn a b = min a b
       minMatch :: [Ident] -> Exp -> Int
+      minMatch _ (Lam i _) | i `elem` as = 0   -- name capture, so ignore matches
       minMatch _ (Lam _ e) = minMatch [] e
       minMatch vs (App f (Var v)) = minMatch (v:vs) f
       minMatch _ (App f a) = minMatch [] f `minn` minMatch [] a
@@ -583,9 +605,57 @@ lazier def@(fcn, l@(Lam _ (Lam _ _))) =
       repl vs (App f a) = app vs $ App (repl [] f) (repl [] a)
       repl [] (Var v) = Var v
       repl vs (Var v)
-        | v == fcn && take arity vs == drops = app (drop arity vs) $ vfcn'
+        | v == fcn && take arity vs == drops = app (drop arity vs) vfcn'
       repl vs e = app vs e
   in  if arity > 0
       then (fcn, lams drops $ letRecE fcn' (lams keeps (repl [] body)) vfcn')
       else def
 lazier def = def
+
+---------
+
+-- "[static] [name.h] [&] [expr]"
+-- "[static] [name.h] value [expr]"
+-- "dynamic"
+-- "wrapper"
+-- When the calling convention is ccall the 'expr' has to be a name,
+-- with capi it can be any C expression.
+parseImpEnt :: SLoc -> CallConv -> String -> ImpEnt
+parseImpEnt _ Cjavascript s = ImpJS s
+parseImpEnt loc _cc s =
+  case words s of
+    ["dynamic"] -> ImpDynamic
+    ["wrapper"] -> ImpWrapper
+    "static" : r -> rest r
+    r            -> rest r
+ where rest (inc : r) | ".h" `isSuffixOf` inc = rest' (ImpStatic [inc]) r
+       rest r                                 = rest' (ImpStatic [])    r
+       rest' c ("&"     : r) = rest'' (c IPtr) r
+       rest' c ['&'     : r] = rest'' (c IPtr) [r]
+       rest' c ("value" : r) = rest'' (c IValue) [unwords r]
+       rest' c r             = rest'' (c IFunc) r
+       rest'' c [n] = c n
+       rest'' _ _ = badForImp loc
+
+badForImp :: SLoc -> a
+badForImp loc = errorMessage loc "bad foreign import"
+
+mkForImp :: Int -> CallConv -> Maybe String -> Ident -> EType -> Lit
+mkForImp _ Cjavascript Nothing i _ = badForImp (getSLoc i)
+mkForImp no cc ms i ty =
+  let cty = CType ty
+      loc = getSLoc i
+      ui  = unIdent (unQualIdent i)
+      isValidC (c:cs) = isAlpha c && all (\ d -> isAlphaNum d || d == '_') cs
+      isValidC _ = False
+  in  case ms of
+        Nothing -> LForImp (ImpStatic [] IFunc ui) ui cty
+        Just s  ->
+          let impent = parseImpEnt loc cc s
+              fno = show no
+              cid =
+                case impent of
+                  ImpStatic _ _ n ->
+                    if isValidC n then n else fno
+                  _ -> fno
+          in  LForImp impent cid cty
