@@ -1,20 +1,20 @@
 {-# OPTIONS_GHC -Wno-unused-imports #-}
-{-# LANGUAGE PatternSynonyms #-}
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module MicroHs.Exp(
   Exp(..),
   PrimOp,
   substExp,
-  app2, app3, cFlip,
+  app2, app3,
   allVarsExp, freeVars,
   lams, apps,
+  mkForExp, getForExp,
   ) where
-import Prelude(); import MHSPrelude hiding((<>))
+import qualified Prelude(); import MHSPrelude hiding((<>))
 import Data.Char
 import Data.List
 import MicroHs.Ident
-import MicroHs.Expr(Lit(..), showLit)
+import MicroHs.Expr(Lit(..), showLit, CType(..))
 import MicroHs.List
 import Text.PrettyPrint.HughesPJLite
 import Debug.Trace
@@ -28,18 +28,18 @@ data Exp
   | Lit Lit
   deriving (Eq)
 
+instance NFData Exp where
+  rnf (Var a) = rnf a
+  rnf (App a b) = rnf a `seq` rnf b
+  rnf (Lam a b) = rnf a `seq` rnf b
+  rnf (Lit a) = rnf a
+
 app2 :: Exp -> Exp -> Exp -> Exp
 app2 f a1 a2 = App (App f a1) a2
 
 app3 :: Exp -> Exp -> Exp -> Exp -> Exp
 app3 f a1 a2 a3 = App (app2 f a1 a2) a3
 
-cFlip :: Exp
-cFlip = Lit (LPrim "C")
-
---cR :: Exp
---cR = Lit (LPrim "R")
-
 instance Show Exp where
   show = render . ppExp
 
@@ -59,13 +59,13 @@ substExp si se ae =
     App f a -> App (substExp si se f) (substExp si se a)
     Lam i e -> if si == i then
                  ae
-               else if elem i (freeVars se) then
+               else if i `elem` freeVars se then
                  let
                    fe = allVarsExp e
                    ase = allVarsExp se
                    j = head [ v | n <- enumFrom (0::Int),
                               let { v = mkIdent ("a" ++ show n) },
-                              not (elem v ase), not (elem v fe), v /= si ]
+                              v `notElem` ase, v `notElem` fe, v /= si ]
                  in
                    --trace ("substExp " ++ show [si, i, j]) $
                    Lam j (substExp si se (substExp i (Var j) e))
@@ -96,3 +96,12 @@ lams xs e = foldr Lam e xs
 apps :: Exp -> [Exp] -> Exp
 apps f = foldl App f
 
+-- Encoding for 'foreign export' definitions.
+mkForExp :: Exp -> CType -> Exp
+mkForExp e t = app2 cfe e cty
+  where cty = Lit $ LCType t
+        cfe = Lit $ LPrim "FE"
+
+getForExp :: Exp -> Maybe (Exp, CType)
+getForExp (App (App (Lit (LPrim "FE")) e) (Lit (LCType t))) = Just (e, t)
+getForExp _ = Nothing
