@@ -4,7 +4,7 @@ module MicroHs.IntSet(
   IntSet,
   empty, member, insert, fromList, toList
   ) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
 import qualified MicroHs.IntMap as M
 
 newtype IntSet = I (M.IntMap ())
@@ -22,8 +22,7 @@ insert :: Int -> IntSet -> IntSet
 insert k (I m) = I (M.insert k () m)
 
 fromList :: [Int] -> IntSet
-fromList is = I (M.fromList (zip is (repeat ())))
+fromList is = I $ M.fromList (map (\ x -> (x, ())) is)
 
 toList :: IntSet -> [Int]
 toList (I m) = map fst (M.toList m)
-
