@@ -5,9 +5,10 @@ module MicroHs.Lex(
   popLayout, lex,
   readInt,
   ) where
-import Prelude(); import MHSPrelude hiding(lex)
+import qualified Prelude(); import MHSPrelude hiding(lex)
 import Data.Char
 import Data.List
+import Data.Maybe (fromJust)
 import MicroHs.Ident
 import Text.ParserComb(TokenMachine(..))
 
@@ -17,12 +18,14 @@ data Token
   | TChar   SLoc Char             -- Char literal
   | TInt    SLoc Integer          -- Integer literal
   | TRat    SLoc Rational         -- Rational literal (i.e., decimal number)
-  | TSpec   SLoc Char             -- one of ()[]{},`;
+  | TSpec   SLoc Char             -- one of ()[]{},`<>;
                                   -- for synthetic {} we use <>, also
                                   --  .  for record selection
                                   --  ~  for lazy
                                   --  !  for strict
                                   --  NOT YET  @  for type app
+                                  --  L  for (#
+                                  --  R  for #)
   | TError  SLoc String           -- lexical error
   | TBrace  SLoc                  -- {n} in the Haskell report
   | TIndent SLoc                  -- <n> in the Haskell report
@@ -39,6 +42,8 @@ showToken (TInt _ i) = show i
 showToken (TRat _ d) = show d
 showToken (TSpec _ c) | c == '<' = "{ layout"
                       | c == '>' = "} layout"
+                      | c == 'L' = "(#"
+                      | c == 'R' = "#)"
                       | otherwise = [c]
 showToken (TError _ s) = s
 showToken (TBrace _) = "TBrace"
@@ -53,8 +58,9 @@ incrLine (SLoc f l _) = let l' = l+1 in seq l' (SLoc f l' 1)
 addCol :: SLoc -> Int -> SLoc
 addCol (SLoc f l c) i = let c' = c+i in seq c' (SLoc f l c')
 
+-- Columns are numbered from 1, so tabs stop are 1, 9, 17, ...
 tabCol :: SLoc -> SLoc
-tabCol (SLoc f l c) = SLoc f l (((c + 7) `quot` 8) * 8)
+tabCol (SLoc f l c) = SLoc f l (((c + 7) `quot` 8) * 8 + 1)
 
 mkLocEOF :: SLoc
 mkLocEOF = SLoc "" (-1) 0
@@ -77,23 +83,34 @@ lex loc ('-':'-':cs) | isComm rs = skipLine (addCol loc $ 2+length ds) cs
     isComm [] = True
     isComm (d:_) = not (isOperChar d)
 lex loc (d:cs) | isLower_ d =
-  case span isIdentChar cs of
+  case spanIdent cs of
     (ds, rs) -> tIdent loc [] (d:ds) (lex (addCol loc $ 1 + length ds) rs)
 lex loc cs@(d:_) | isUpper d = upperIdent loc loc [] cs
-lex loc ('0':x:cs) | toLower x == 'x' = hexNumber loc cs
-                   | toLower x == 'o' = octNumber loc cs
-                   | toLower x == 'b' = binNumber loc cs
-lex loc cs@(d:_) | isDigit d = number loc cs
+lex loc ('0':x:cs)
+  | toLower x == 'x' = lexNumBasePrefix x 16 isHexDigit loc cs
+  | toLower x == 'o' = lexNumBasePrefix x  8 isOctDigit loc cs
+  | toLower x == 'b' = lexNumBasePrefix x  2 isBinDigit loc cs
+  where isBinDigit c = c == '0' || c == '1'
+lex loc cs@(d:_) | isDigit d =
+  case readNumDec cs of
+    (Left n,  len, rs) -> TInt loc n : lexSkipHash (addCol loc len) rs
+    (Right q, len, rs) -> TRat loc q : lexSkipHash (addCol loc len) rs
 lex loc ('.':cs@(d:_)) | isLower_ d =
   TSpec loc '.' : lex (addCol loc 1) cs
+lex loc ('(':dcs@(d:cs)) | d == '#'  = TSpec loc 'L' : lex (addCol loc 2) cs
+                         | otherwise = TSpec loc '(' : lex (addCol loc 1) dcs
+lex loc ('#':')':cs) = TSpec loc 'R' : lex (addCol loc 2) cs
 -- Recognize #line 123 "file/name.hs"
 lex loc ('#':xcs) | (SLoc _ _ 1) <- loc, Just cs <- stripPrefix "line " xcs =
   case span (/= '\n') cs of
     (line, rs) ->        -- rs will contain the '\n', so subtract 1 below
       let ws = words line
-          file = tail $ init $ ws!!1   -- strip the initial and final '"' 
+          file = tail $ init $ ws!!1   -- strip the initial and final '"'
           loc' = SLoc file (readInt (ws!!0) - 1) 1
       in  lex loc' rs
+                  | (SLoc _ 1 1) <- loc, take 1 xcs == "!" =
+  -- It's a shebang (#!), ignore the rest of the line
+  skipLine loc xcs
 lex loc ('!':' ':cs) =  -- ! followed by a space is always an operator
   TIdent loc [] "!" : lex (addCol loc 2) cs
 lex loc (c:cs@(d:_)) | isSpecSing c && not (isOperChar d) = -- handle reserved
@@ -103,15 +120,20 @@ lex loc (c:cs@(d:_)) | isSpecSing c && not (isOperChar d) = -- handle reserved
 lex loc (d:cs) | isOperChar d =
   case span isOperChar cs of
     (ds, rs) -> TIdent loc [] (d:ds) : lex (addCol loc $ 1 + length ds) rs
-lex loc (d:cs) | isSpec d  = TSpec loc d : lex (addCol loc 1) cs
-lex loc ('"':cs) =
-  case takeChars loc (TString loc) '"' (addCol loc 1) [] cs of
-    (t, loc', rs) -> t : lex loc' rs
-lex loc ('\'':cs) =
-  let tchar [c] = TChar loc c
-      tchar _ = TError loc "Illegal Char literal"
-  in  case takeChars loc tchar '\'' (addCol loc 1) [] cs of  -- XXX head of
-        (t, loc', rs) -> t : lex loc' rs
+lex loc (d:cs) | isSpec d =
+  TSpec loc d : lex (addCol loc 1) cs
+lex loc ('"':'"':'"':cs) = lexLitStr loc (addCol loc 3) (TString loc) isTrip multiLine cs
+  where isTrip ('"':'"':'"':_) = Just 3
+        isTrip _ = Nothing
+lex loc ('"':cs) = lexLitStr loc (addCol loc 1) (TString loc) isDQuote id cs
+  where isDQuote ('"':_) = Just 1
+        isDQuote _ = Nothing
+lex loc ('\'':cs) = lexLitStr loc (addCol loc 1) tchar isSQuote id cs
+  where isSQuote ('\'':_) = Just 1
+        isSQuote _ = Nothing
+        tchar [c] = TChar loc c
+        tchar _   = TError loc "Illegal Char literal"
+
 lex loc (d:_) = [TError loc $ "Unrecognized input: " ++ show d]
 lex loc [] = [TEnd loc]
 
@@ -119,42 +141,69 @@ nested :: SLoc -> [Char] -> [Token]
 nested loc ('#':cs) = pragma loc cs
 nested loc cs = skipNest loc 1 cs
 
-hexNumber :: SLoc -> String -> [Token]
-hexNumber loc cs =
-  case span isHexDigit cs of
-    (ds, rs) -> TInt loc (readBase 16 ds) : lex (addCol loc $ length ds + 2) rs
+-- lex a number of the form '0':x:cs
+lexNumBasePrefix :: Char -> Integer -> (Char -> Bool) -> SLoc -> String -> [Token]
+lexNumBasePrefix x base isDig loc cs =
+  case readIntBase base isDig cs of
+    Just (n, len, rs) -> TInt loc n : lexSkipHash (addCol loc $ len + 2) rs
+    Nothing           -> TInt loc 0 : lexSkipHash (addCol loc 1) (x : cs)
 
-octNumber :: SLoc -> String -> [Token]
-octNumber loc cs =
-  case span isOctDigit cs of
-    (ds, rs) -> TInt loc (readBase 8 ds) : lex (addCol loc $ length ds + 2) rs
+-- Used to skip # after numbers
+lexSkipHash :: SLoc -> String -> [Token]
+lexSkipHash loc ('#':cs) = lexSkipHash (addCol loc 1) cs
+lexSkipHash loc cs = lex loc cs
 
-binNumber :: SLoc -> String -> [Token]
-binNumber loc cs =
-  case span isBinDigit cs of
-    (ds, rs) -> TInt loc (readBase 2 ds) : lex (addCol loc $ length ds + 2) rs
-  where isBinDigit c = c == '0' || c == '1'
-
-number :: SLoc -> String -> [Token]
-number loc cs =
-  case span isDigit cs of
-    (ds, rs) | null rs || not (head rs == '.') || (take 2 rs) == ".." ->
-               case expo rs of
-                 Nothing -> TInt loc (readBase 10 ds) : lex (addCol loc $ length ds) rs
-                 Just (es, rs') -> mkD (ds ++ es) rs'
-             | otherwise ->
-               case span isDigit (tail rs) of
-                 (ns, rs') ->
-                   let s = ds ++ '.':ns
-                   in  case expo rs' of
-                         Nothing -> mkD s rs'
-                         Just (es, rs'') -> mkD (s ++ es) rs''
+readIntBase :: Integer -> (Char -> Bool) -> String -> Maybe (Integer, Int, String)
+readIntBase base isDig ds =
+    let (n, len, rest) = goDig 0 0 ds
+    in if len > 0 then Just (n, len, rest) else Nothing
   where
-    expo (e:'-':xs@(d:_)) | toLower e == 'e' && isDigit d = Just ('e':'-':as, bs) where (as, bs) = span isDigit xs
-    expo (e:'+':xs@(d:_)) | toLower e == 'e' && isDigit d = Just ('e':'+':as, bs) where (as, bs) = span isDigit xs
-    expo (e:    xs@(d:_)) | toLower e == 'e' && isDigit d = Just ('e':    as, bs) where (as, bs) = span isDigit xs
-    expo _ = Nothing
-    mkD x r = TRat loc (readRational x) : lex (addCol loc $ length x) r
+    goSep acc lastLen lastRest len ('_' : cs) = goSep acc lastLen lastRest (len + 1) cs
+    goSep acc _ _ len (d : cs) | isDig d = goDig (addDigit acc d) (len + 1) cs
+    goSep acc lastLen lastRest _ _ = (acc, lastLen, lastRest)
+
+    goDig acc len rest@('_' : cs) = goSep acc len rest (len + 1) cs
+    goDig acc len (d : cs) | isDig d = goDig (addDigit acc d) (len + 1) cs
+    goDig acc len rest = (acc, len, rest)
+
+    addDigit x d = x * base + toInteger (digitToInt d)
+
+readNumDec :: String -> (Either Integer Rational, Int, String)
+readNumDec cs =
+  case readIntDec cs of
+    Just (n, nLen, rest) ->
+      case rest of
+        '.' : rs@(d : _) | isDigit d ->
+          case readIntDec rs of
+            Just (m, mLen, rest') ->
+              let q = toRational n + toRational m * 10 ^^ negate (length $ filter isDigit $ take mLen rs)
+              in case expo rest' of
+                Just (e, eLen, rest'') -> (Right $ q * 10 ^^ e, nLen + 1 + mLen + eLen, rest'')
+                Nothing -> (Right q, nLen + 1 + mLen, rest')
+            Nothing -> (Left n, nLen, rest) -- this can't happen
+        _ ->
+          case expo rest of
+            Just (e, eLen, rest') -> (Right $ toRational n * 10 ^^ e, nLen + eLen, rest')
+            Nothing -> (Left n, nLen, rest)
+    Nothing -> error "impossible: first char is a digit"
+  where
+    readIntDec = readIntBase 10 isDigit
+
+    -- try to read an exponent
+    expo :: String -> Maybe (Integer, Int, String)
+    expo = go 0
+
+    go len ('_' : xs) = go (len + 1) xs
+    go len (e:'-':xs@(d:_)) | toLower e == 'e' && isDigit d =
+      let (n, len', rest) = fromJust $ readIntDec xs
+      in Just (-n, len + 2 + len', rest)
+    go len (e:'+':xs@(d:_)) | toLower e == 'e' && isDigit d =
+      let (n, len', rest) = fromJust $ readIntDec xs
+      in Just (n, len + 2 + len', rest)
+    go len (e:    xs@(d:_)) | toLower e == 'e' && isDigit d =
+      let (n, len', rest) = fromJust $ readIntDec xs
+      in Just (n, len + 1 + len', rest)
+    go _ _ = Nothing
 
 -- Skip a {- -} style comment
 skipNest :: SLoc -> Int -> String -> [Token]
@@ -171,50 +220,60 @@ skipNest loc _ []           = [TError loc "Unclosed {- comment"]
 skipLine :: SLoc -> String -> [Token]
 skipLine loc cs@('\n':_) = lex loc cs
 skipLine loc (_:cs)      = skipLine loc cs
-skipLine   _ []          = []
+skipLine loc []          = lex loc []
 
 -- | Takes a list of tokens and produces a list of tokens. If the first token in
 -- the input list is a TIndent, the input is returned unaltered. Otherwise, a
--- TIndent is prepended to the input list        
+-- TIndent is prepended to the input list
 tIndent :: [Token] -> [Token]
 tIndent ts@(TIndent _ : _) = ts
 tIndent ts = TIndent (tokensLoc ts) : ts
 
-takeChars :: SLoc -> (String -> Token) -> Char -> SLoc -> String -> String -> (Token, SLoc, String)
-takeChars oloc  _ c loc _ [] = (TError oloc ("Unmatched " ++ [c]), loc, [])
-takeChars oloc fn c loc str ('\\':cs) =
-  let skipGap l (' ' :rs) = skipGap (addCol l 1) rs
-      skipGap l ('\n':rs) = skipGap (incrLine l) rs
-      skipGap l ('\r':rs) = skipGap l rs
-      skipGap l ('\t':rs) = skipGap (tabCol l) rs
-      skipGap l ('\\':rs) = takeChars oloc fn c (addCol l 1) str rs
-      skipGap l       rs  = (TError oloc "Bad string gap", l, rs)
-  in
-  case cs of
-    '&':rs -> takeChars oloc fn c (addCol loc 2) str rs
-    d:_ | isSpace d -> skipGap loc cs
-    _ ->
-      case decodeChar cs of
-        (d, m, rs) -> takeChars oloc fn c (addCol loc m) (d:str) rs
-takeChars   _  fn c loc str (d:cs) | c == d = (fn (reverse str), addCol loc 1, cs)
-takeChars oloc fn c loc str (d:cs) = takeChars oloc fn c (addCol loc 1) (d:str) cs
+lexLitStr :: SLoc -> SLoc -> (String -> Token) -> (String -> Maybe Int) -> (String -> String) -> String -> [Token]
+lexLitStr oloc loc mk end post acs = loop loc [] acs
+  where loop l rs cs | Just k <- end cs   = mk (decodeEscs $ post $ reverse rs) : lex (addCol l k) (drop k cs)
+        loop l rs ('\\':c:cs) | isSpace c = remGap l rs cs
+        loop l rs ('\\':'^':'\\':cs)      = loop (addCol l 3) ('\\':'^':'\\':rs) cs  -- special hack for unescaped \
+        loop l rs ('\\':cs)               = loop' (addCol l 1) ('\\':rs) cs
+        loop l rs       cs                = loop' l rs cs
 
-decodeChar :: String -> (Char, Int, String)
-decodeChar ('n':cs) = ('\n', 1, cs)
-decodeChar ('a':cs) = ('\a', 1, cs)
-decodeChar ('b':cs) = ('\b', 1, cs)
-decodeChar ('f':cs) = ('\f', 1, cs)
-decodeChar ('r':cs) = ('\r', 1, cs)
-decodeChar ('t':cs) = ('\t', 1, cs)
-decodeChar ('v':cs) = ('\v', 1, cs)
-decodeChar ('x':cs) = conv 16 1 0 cs
-decodeChar ('o':cs) = conv 8 1 0 cs
-decodeChar ('^':c:cs) | '@' <= c && c <= '_' = (chr (ord c - ord '@'), 2, cs)
-decodeChar (cs@(c:_)) | isDigit c = conv 10 0 0 cs
-decodeChar (c1:c2:c3:cs) | Just c <- lookup [c1,c2,c3] ctlCodes = (c, 3, cs)
-decodeChar (c1:c2:cs) | Just c <- lookup [c1,c2] ctlCodes = (c, 2, cs)
-decodeChar (c  :cs) = (c,    1, cs)
-decodeChar []       = ('X',  0,   [])
+        loop' l rs ('\n' :cs) = loop  (incrLine l) ( '\n':rs) cs
+        loop' l rs ('\t' :cs) = loop  (tabCol   l) ( '\t':rs) cs
+        loop' l rs ('\r' :cs) = loop            l         rs  cs
+        loop' l rs     (c:cs) = loop  (addCol l 1) (    c:rs) cs
+        loop' _ _          [] = [TError oloc "unterminated Char/String literal"]
+--        foo xs = trace (show ("foo", loc, take 20 acs, xs)) xs
+
+        remGap l rs ('\\':cs) = loop   (addCol l 1)       rs  cs
+        remGap l rs ('\n':cs) = remGap (incrLine l) ('\n':rs) cs
+        remGap l rs ('\t':cs) = remGap (tabCol   l) ('\t':rs) cs
+        remGap l rs ('\r':cs) = remGap           l        rs  cs
+        remGap l rs (' ' :cs) = remGap (addCol l 1)       rs  cs
+        remGap l _         _  = --errorMessage oloc "bad string gap"
+                                mhsError (show l ++ ": bad string gap")
+
+decodeEscs :: String -> String
+decodeEscs [] = []
+decodeEscs ('\\':cs) = decodeEsc cs
+decodeEscs (c:cs) = c : decodeEscs cs
+
+decodeEsc :: String -> String
+decodeEsc ('n':cs) = '\n' : decodeEscs cs
+decodeEsc ('a':cs) = '\a' : decodeEscs cs
+decodeEsc ('b':cs) = '\b' : decodeEscs cs
+decodeEsc ('f':cs) = '\f' : decodeEscs cs
+decodeEsc ('r':cs) = '\r' : decodeEscs cs
+decodeEsc ('t':cs) = '\t' : decodeEscs cs
+decodeEsc ('v':cs) = '\v' : decodeEscs cs
+decodeEsc ('&':cs) = decodeEscs cs
+decodeEsc ('x':cs) = conv 16 0 cs
+decodeEsc ('o':cs) = conv 8 0 cs
+decodeEsc ('^':c:cs) | '@' <= c && c <= '_' = chr (ord c - ord '@') : decodeEscs cs
+decodeEsc cs@(c:_) | isDigit c = conv 10 0 cs
+decodeEsc (c1:c2:c3:cs) | Just c <- lookup [c1,c2,c3] ctlCodes = c : decodeEscs cs
+decodeEsc (c1:c2:cs) | Just c <- lookup [c1,c2] ctlCodes = c : decodeEscs cs
+decodeEsc (c  :cs) = c : decodeEscs cs
+decodeEsc []       = mhsError "Bad \\ escape"
 
 -- Nobody uses these, but it's part of the Haskell Report so...
 ctlCodes :: [(String, Char)]
@@ -222,16 +281,46 @@ ctlCodes =
   [("NUL", '\NUL'), ("SOH", '\SOH'), ("STX", '\STX'), ("ETX", '\ETX'),
    ("EOT", '\EOT'), ("ENQ", '\ENQ'), ("ACK", '\ACK'), ("BEL", '\BEL'),
    ("BS",  '\BS'),  ("HT",  '\HT'),  ("LF",  '\LF'),  ("VT", '\VT'),
-   ("FF",  '\FF'),  ("CR",  '\CR'),  ("SO",  '\SO'),  ("SI", '\SI'), 
+   ("FF",  '\FF'),  ("CR",  '\CR'),  ("SO",  '\SO'),  ("SI", '\SI'),
    ("DLE", '\DLE'), ("DC1", '\DC1'), ("DC2", '\DC2'), ("DC3", '\DC3'),
    ("DC4", '\DC4'), ("NAK", '\NAK'), ("SYN", '\SYN'), ("ETB", '\ETB'),
    ("CAN", '\CAN'), ("EM",  '\EM'),  ("SUB", '\SUB'), ("ESC", '\ESC'),
-   ("FS",  '\FS'),  ("GS",  '\GS'),  ("RS",  '\RS'),  ("US",  '\US'), 
+   ("FS",  '\FS'),  ("GS",  '\GS'),  ("RS",  '\RS'),  ("US",  '\US'),
    ("SP",  '\SP'),  ("DEL", '\DEL')]
 
-conv :: Int -> Int -> Int -> String -> (Char, Int, String)
-conv b k r (c:ds) | isHexDigit c, let { n = digitToInt c }, n < b = conv b (k+1) (r * b + n) ds
-conv _ k r ds = (chr r, k, ds)
+conv :: Int -> Int -> String -> String
+conv b r (c:ds) | isHexDigit c && n < b = conv b (r * b + n) ds
+  where n = digitToInt c
+conv _ r ds = chr r : decodeEscs ds
+
+-- Multiline string literals
+multiLine :: String -> String
+multiLine =
+  finalTrim          .     -- trim initial \n
+  intercalate "\\n"  .     -- join with \n
+  map removeAllWhite .     -- remove white-space only
+  removeCommonPrefix .     -- remove common space prefix
+  map tabToSpace     .     -- replace leading tabs with spaces
+  lines                    -- split the string by newlines
+  where
+    tabToSpace = to 0
+      where to n ('\t':cs) = replicate (8 - n `rem` 8) ' ' ++ to 0 cs
+            to n (' ' :cs) = ' ' : to (n+1) cs
+            to _ cs        = cs
+    removeCommonPrefix :: [String] -> [String]
+    removeCommonPrefix [] = []
+    removeCommonPrefix (l:ls) = l : map (drop k) ls
+      where k = foldl' pref 1000000 ls
+            pref n [] = n
+            pref n cs =
+              case span isSpace cs of
+                (_, []) -> n                 -- ignore white space only
+                (w, _)  -> min n (length w)  -- find common prefix length
+    removeAllWhite cs | all isSpace cs = ""
+                      | otherwise      = cs
+    -- The GHC manual is wrong.  Follow the implementation.
+    finalTrim ('\\':'n':cs) = cs
+    finalTrim cs            = cs
 
 -- These characters are single characters token, no matter what.
 isSpec :: Char -> Bool
@@ -264,14 +353,19 @@ upperIdent loc sloc qs acs =
    (ds, rs) ->
     case rs of
       '.':cs@(d:_) | isUpper d    -> upperIdent (addCol loc $ 1 + length ds) sloc (ds:qs) cs
-                   | isLower d    -> ident isIdentChar
-                   | isOperChar d -> ident isOperChar
-         where {
-           ident p =
-             case span p cs of
-               (xs, ys) -> tIdent sloc (reverse (ds:qs)) xs (lex (addCol loc $ 1 + length ds + length xs) ys)
-           }
-      _ -> TIdent sloc (reverse qs) ds : lex (addCol loc $ length ds) rs
+                   | isLower_ d   -> ident (spanIdent cs)
+                   | isOperChar d -> ident (span isOperChar cs)
+         where
+           ident (xs, ys) = tIdent sloc (reverse (ds:qs)) xs (lex (addCol loc $ 1 + length ds + length xs) ys)
+      '#':_ -> mk (ds ++ hs) rs' where (hs, rs') = span (== '#') rs
+      _ -> mk ds rs
+  where
+     mk ds rs = TIdent sloc (reverse qs) ds : lex (addCol loc $ length ds) rs
+
+spanIdent :: String -> (String, String)
+spanIdent s = mk $ span isIdentChar s
+  where mk (ds, '#':rs) = mk (ds ++ "#", rs)
+        mk r = r
 
 -- For LambdaCase
 tLam :: [Token] -> [Token]
@@ -279,12 +373,13 @@ tLam (t@(TIdent _ [] "case") : ts) = t : tBrace ts
 tLam ts = ts
 
 tIdent :: SLoc -> [String] -> String -> [Token] -> [Token]
-tIdent loc qs kw ats | elem kw ["let", "where", "do", "of"]
+tIdent loc qs kw ats | elem kw ["let", "where", "do", "of", "mdo"]
                        || kw == "if" && isBar ats                -- For MultiWayIf
                                  = ti : tBrace ats
                      | otherwise = ti : ats
   where
     isBar (TSpec _ '|' : _) = True
+    isBar (TIndent _ : TSpec _ '|' : _) = True
     isBar _ = False
 
     ti = TIdent loc qs kw
@@ -318,10 +413,14 @@ readInt = fromInteger . readBase 10
 -- XXX This is a pretty hacky recognition of pragmas.
 pragma :: SLoc -> [Char] -> [Token]
 pragma loc cs =
-  let as = map toUpper $ takeWhile isAlpha $ dropWhile isSpace cs
-      skip = skipNest loc 1 ('#':cs)
-  in  case as of
-        "SOURCE" -> TPragma loc as : skip
+  let skip = skipNest loc 1 ('#':cs)
+  in  case words cs of
+        p : _ | map toUpper p == "SOURCE" -> TPragma loc p : skip
+        -- hsc2hs generates LINE pragmas
+        p : ln@(_:_) : fn : _ | map toUpper p == "LINE", all isDigit ln ->
+          let f = tail (init fn)
+              l = readInt ln - 1
+          in  seq l $ skipNest (SLoc f l 1) 1 ('#':cs)
         _ -> skip
 
 -- | This is the magical layout resolver, straight from the Haskell report.
@@ -337,20 +436,22 @@ pragma loc cs =
 --   Pop   pop the context stack
 --   Raw   return the rest of the tokens, unprocessed
 
-newtype LexState = LS (Cmd -> (Token, LexState))
+data LexState = LS (Cmd -> (Token, LexState))
 
 data Cmd = Next | Raw | Pop
 
 layoutLS ::                [Token] ->    [Int] -> Cmd      -> (Token,                    LexState)
 layoutLS                        ts           ms  Raw        = (TRaw ts,                  LS $ layoutLS  ts     ms )
-layoutLS                        ts          mms  Pop        =                                                    
-                                                   case (mms, ts) of                                              
+layoutLS                        ts          mms  Pop        =
+                                                   case (mms, ts) of
                                                      (m:ms,_:_) | m/=0 -> (TEnd (tokensLoc ts),  LS $ layoutLS  ts     ms )
                                                      _ ->     (TError l "syntax error",  LS $ layoutLS  []     [] ) where l = tokensLoc ts
 -- The rest are the Next commands
 layoutLS tts@(TIndent x       : ts) mms@(m : ms) _ | n == m = (TSpec (tokensLoc ts) ';', LS $ layoutLS  ts    mms )
                                                    | n <  m = (TSpec (tokensLoc ts) '>', LS $ layoutLS tts     ms ) where {n = getCol x}
 layoutLS     (TIndent _       : ts)          ms  _          =                                 layoutLS  ts     ms  Next
+layoutLS     (t@(TIdent _ _ "do") :  -- for NondecreasingIndentation
+              TBrace x        : ts) mms@(m :  _) _ | n >= m = (t                       , LS $ layoutLS  (TSpec (tokensLoc ts) '<' : ts) (n:mms)) where {n = getCol x}
 layoutLS     (TBrace x        : ts) mms@(m :  _) _ | n > m  = (TSpec (tokensLoc ts) '<', LS $ layoutLS  ts (n:mms)) where {n = getCol x}
 layoutLS     (TBrace x        : ts)          []  _ | n > 0  = (TSpec (tokensLoc ts) '<', LS $ layoutLS  ts     [n]) where {n = getCol x}
 layoutLS     (TBrace x        : ts)          ms  _          = (TSpec (tokensLoc ts) '<', LS $ layoutLS  (TSpec (tokensLoc ts) '>' : TIndent x : ts) ms)
@@ -386,30 +487,3 @@ lexStart ts =
 lexTopLS :: FilePath -> String -> LexState
 lexTopLS f s = LS $ layoutLS (lexStart $ lex (SLoc f 1 1) s) []
   -- error $ show $ map showToken $ lex (SLoc f 1 1) s
-
------------
-
--- Convert string in scientific notation to a rational number.
-readRational :: String -> Rational
-readRational "" = undefined
-readRational acs@(sgn:as) | sgn == '-' = negate $ rat1 as
-                          | otherwise  =          rat1 acs
-  where
-    rat1 s1 =
-      case span isDigit s1 of
-        (ds1, cr1) | ('.':r1) <- cr1                   -> rat2 f1 r1
-                   | (c:r1)   <- cr1, toLower c == 'e' -> rat3 f1 r1
-                   | otherwise                         -> f1
-          where f1 = toRational (readBase 10 ds1)
-
-    rat2 f1 s2 =
-      case span isDigit s2 of
-        (ds2, cr2) | (c:r2) <- cr2, toLower c == 'e' -> rat3 f2 r2
-                   | otherwise                       -> f2
-          where f2 = f1 + toRational (readBase 10 ds2) * 10 ^^ (negate $ length ds2)
-
-    rat3 f2 ('+':s) = f2 * expo s
-    rat3 f2 ('-':s) = f2 / expo s
-    rat3 f2      s  = f2 * expo s
-
-    expo s = 10 ^ (readBase 10 s)
