@@ -1,7 +1,7 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
 module MicroHs.MakeCArray(makeCArray) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
 import Data.Char
 import MicroHs.Flags
 import qualified System.Compress as C
@@ -25,9 +25,9 @@ makeCArray flags file =
 makeCArray' :: String -> String
 makeCArray' file =
   let chunks = chunkify 20 file
-  in  unlines $ ["static unsigned char data[] = {"] ++
+  in  unlines $ ["static const unsigned char data[] = {"] ++
                 map showChunk chunks ++
                 ["};",
-                 "unsigned char *combexpr = data;",
-                 "int combexprlen = " ++ show (length file) ++ ";"
+                 "const unsigned char *combexpr = data;",
+                 "const int combexprlen = " ++ show (length file) ++ ";"
                 ]
