@@ -1,8 +1,9 @@
 module MicroHs.Package(
   IdentPackage,
   Package(..),
+  forcePackage,
   ) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
 import Data.Version
 import MicroHs.Desugar(LDef)
 import MicroHs.Ident(Ident)
@@ -21,7 +22,7 @@ import MicroHs.TypeCheck(TModule, GlobTables)
 --   Foo/Bar.txt
 --   Foo/Baz.txt
 -- The files Foo/Bar.txt and Foo/Baz.txt will contain simply "foo.pkg".
--- 
+--
 
 type IdentPackage = Ident
 
@@ -29,9 +30,17 @@ data Package = Package {
   pkgName      :: IdentPackage,                    -- package name
   pkgVersion   :: Version,                         -- package version
   pkgCompiler  :: String,                          -- compiler version that created the package
+  pkgOptl      :: [String],                        -- linker options passed by mhs to the c compiler
   pkgExported  :: [TModule [LDef]],                -- exported modules
   pkgOther     :: [TModule [LDef]],                -- non-exported modules
   pkgTables    :: GlobTables,                      -- global tables
   pkgDepends   :: [(IdentPackage, Version)]        -- used packages
   }
   -- deriving (Show)
+
+instance NFData Package where
+  rnf (Package a b c d e f g h) = rnf a `seq` rnf b `seq` rnf c `seq` rnf d `seq` rnf e `seq` rnf f `seq` rnf g `seq` rnf h
+
+-- Fully evaluate a package
+forcePackage :: Package -> Package
+forcePackage p = force p
