@@ -1,6 +1,6 @@
 -- Copyright 2023 Lennart Augustsson
 -- See LICENSE file for full license.
-{-# OPTIONS_GHC -Wno-unused-imports -Wno-dodgy-imports #-}
+{-# OPTIONS_GHC -Wno-unused-imports -Wno-dodgy-imports -Wno-noncanonical-monad-instances #-}
 -- State monad over IO
 module MicroHs.StateIO(
   module MicroHs.StateIO,
@@ -8,17 +8,16 @@ module MicroHs.StateIO(
   module Control.Monad,
   module Data.Functor,
   ) where
-import Prelude(); import MHSPrelude
+import qualified Prelude(); import MHSPrelude
 import Control.Applicative
+import Control.Exception
 import Control.Monad
 import Data.Functor hiding(unzip)
 
-data StateIO s a = S (s -> IO (a,s))
+newtype StateIO s a = S (s -> IO (a,s))
 
 runStateIO :: forall s a . StateIO s a -> (s -> IO (a,s))
-runStateIO sa =
-  case sa of
-    S x -> x
+runStateIO (S x) = x
 
 execStateIO :: forall s a . StateIO s a -> s -> IO s
 execStateIO sa s = do
@@ -26,26 +25,27 @@ execStateIO sa s = do
   case as of
     (_, ss) -> return ss
 
-instance forall s . Functor (StateIO s) where
+instance Functor (StateIO s) where
   fmap f sa = S $ \ s -> do
     (a, ss) <- runStateIO sa s
     return (f a, ss)
 
-instance forall s . Applicative (StateIO s) where
+instance Applicative (StateIO s) where
   pure a = S $ \ s -> return (a, s)
   (<*>) = ap
-  (*>) m k = S $ \ s -> do
-    (_, ss) <- runStateIO m s
-    runStateIO k ss
+  -- Hugs doesn't have *> here
 
-instance forall s . Monad (StateIO s) where
+instance Monad (StateIO s) where
   (>>=) m k = S $ \ s -> do
     (a, ss) <- runStateIO m s
     runStateIO (k a) ss
-  (>>) = (*>)
+  (>>) m k = S $ \ s -> do
+    (_, ss) <- runStateIO m s
+    runStateIO k ss
+  return = pure
 
 {-
-instance forall s . MonadFail (StateIO s) where
+instance MonadFail (StateIO s) where
   fail = error
 -}
 
@@ -65,3 +65,10 @@ liftIO :: forall s a . IO a -> StateIO s a
 liftIO io = S $ \ s -> do
   a <- io
   return (a, s)
+
+trySIO :: StateIO s a -> StateIO s (Either SomeException a)
+trySIO (S io) = S $ \ s -> do
+  r <- try (io s)
+  case r of
+    Left e -> return (Left e, s)
+    Right (a, s') -> return (Right a, s')
