@@ -1,5 +1,5 @@
 module Array where
-import Data.Array
+import Mhs.Array
 
 main :: IO ()
 main = do
@@ -19,4 +19,3 @@ main = do
   print $ a // [(1,'w')]
   print $ fmap fromEnum a
   print $ ixmap (0,2) succ a
-  
\ No newline at end of file
