@@ -1,16 +1,15 @@
 module Eq(main) where
-import Prelude
 
 main :: IO ()
 main = do
-  putStrLn $ show [1==(1::Int), 'a'=='a', 1.1==(1.1::Double),
+  print [1==(1::Int), 'a'=='a', 1.1==(1.1::Double),
                    True==True, False==False,
                    (Nothing::Maybe Int)==Nothing, Just (1::Int) == Just 1,
                    [1,2,3] == [1,2,3::Int],
                    (1,2) == (1::Int,2::Int),
                    (Left 1 :: Either Int Char) == Left 1, (Right 'a' :: Either Int Char) == Right 'a'
                   ]
-  putStrLn $ show [1==(2::Int), 'a'=='b', 1.1==(1.2::Double),
+  print [1==(2::Int), 'a'=='b', 1.1==(1.2::Double),
                    True==False, False==True,
                    Nothing==Just (1::Int), Just (1::Int) == Nothing,
                    [1,2,3] == [1,2,4::Int],
