@@ -1,5 +1,4 @@
 module Exists(main) where
-import Prelude
 
 data E = forall a . Show a => C a
 
@@ -31,6 +30,20 @@ data T a = Show a => T a
 sh :: forall a . T a -> String
 sh (T a) = show a
 
+data D = forall a . (Show a, Num a) => D a
+
+unD :: D -> String
+unD (D x) = show (x+1)
+
+scp :: Show a => a -> String
+scp a =
+  let t = C a
+      tt = case t of
+             C (x :: a) -> C ([] :: [a])
+             --C x -> C ([] `asTypeOf` [x])
+  in  case tt of
+        C y -> show y
+
 main :: IO ()
 main = do
   print es
@@ -40,3 +53,7 @@ main = do
   putStrLn (bar someE)
   putStrLn (sh (T True))
   putStrLn (sh (T 'x'))
+  putStrLn (unD (D (1::Int)))
+  putStrLn (unD (D 1.5))
+  putStrLn (scp 'a')
+  
