@@ -1,11 +1,9 @@
 module ForeignPtr(main) where
+import System.Mem
 import Foreign.Ptr
 import Foreign.ForeignPtr
 import Foreign.Storable
 
-gc :: IO ()
-gc = primitive "IO.gc"
-
 add :: Ptr a -> Int -> Ptr a
 add = plusPtr
 
@@ -19,11 +17,11 @@ main = do
     poke p (42::Int)
     poke (add p sInt) (88::Int)
   withForeignPtr fp $ \ p -> do
-    gc
+    performGC
     peek p >>= print
     peek (add p sInt) >>= print
   let fp1 :: ForeignPtr Int
       fp1 = plusForeignPtr fp sInt
   withForeignPtr fp1 $ \ p -> do
     peek p >>= print
-  gc
+  performGC
