@@ -1,8 +1,7 @@
 module Guard(main) where
-import Prelude
 
 f :: [Int] -> Int
-f [x] | x < 0 = 0 - x
+f [x] | x < 0 = - x
       | x < 10 = x+1
       | otherwise = 99
 f [x,y] = x+y
@@ -12,4 +11,4 @@ f xs | l < 4 = 0
 
 main :: IO ()
 main = do
-  putStrLn $ show [f [0 - 7], f [5], f [20], f [2,3], f [1,2,3], f[1,2,3,4]]
+  print [f [- 7], f [5], f [20], f [2,3], f [1,2,3], f[1,2,3,4]]
