@@ -1,5 +1,4 @@
 module Infix(main) where
-import Prelude
 
 infix 1 ===
 infixl 2 +++
@@ -16,4 +15,4 @@ a &&& b = a * (b + 1)
 
 main :: IO ()
 main = do
-  putStrLn $ show $ 2 +++ 3 &&& 4 === 17
+  print $ 2 +++ 3 &&& 4 === 17
