@@ -1,13 +1,20 @@
 module Info(main) where
-import Prelude
 import Foreign.Storable
 import Foreign.Marshal.Utils
 import Foreign.Ptr
 import Data.Word
 
+foreign import capi "want_gmp" want_gmp :: Int
+
+thisOS :: String
+thisOS | _isWindows = "Windows"
+       | _isMacOS   = "MacOS"
+       | _isLinux   = "Linux"
+       | otherwise  = "Unknown OS"
+
 main :: IO ()
 main = do
-  putStrLn $ "Running on " ++ if _isWindows then "Windows" else "Unix"
+  putStrLn $ "Running on " ++ thisOS
   putStr $ show _wordSize ++ " bit words, "
 
   let
@@ -20,3 +27,5 @@ main = do
       1 -> "big endian"
       2 -> "little endian"
       _ -> "Mystery Endian"
+
+  putStrLn $ "GMP: " ++ if want_gmp /= 0 then "yes" else "no"
