@@ -1,5 +1,4 @@
 module LitMatch(main) where
-import Prelude
 
 f :: Int -> Int
 f 0 = 10
@@ -26,7 +25,7 @@ s _ = 3
 
 main :: IO ()
 main = do
-  putStrLn $ show [f 0, f 1, f 10, f (-1)]
-  putStrLn $ show [g 1 0, g 1 1, g 2 0, g 2 1, g 2 2]
-  putStrLn $ show [h 'a', h 'b', h 'c']
-  putStrLn $ show [s "aaa", s "apa", s "foo"]
+  print [f 0, f 1, f 10, f (-1)]
+  print [g 1 0, g 1 1, g 2 0, g 2 1, g 2 2]
+  print [h 'a', h 'b', h 'c']
+  print [s "aaa", s "apa", s "foo"]
