@@ -1,5 +1,4 @@
 module Newtype(main) where
-import Prelude
 
 newtype N = N Int
 newtype M a = M a
@@ -15,5 +14,5 @@ showM sh (M x) = "(M " ++ sh x ++ ")"
 
 main :: IO ()
 main = do
-  putStrLn $ show [f (N 1), f (N 2)]
+  print [f (N 1), f (N 2)]
   putStrLn $ showM show (g (M (3,4)))
