@@ -1,5 +1,4 @@
 module Rank2(main) where
-import Prelude
 
 f :: (forall a . a -> a) -> (Int, Bool)
 f i = (i (1::Int), i True)
@@ -7,14 +6,14 @@ f i = (i (1::Int), i True)
 g :: (forall a . a -> Int -> a) -> (Int, Bool)
 g c = (c (1::Int) (1::Int), c True (1::Int))
 
-data Id = Id (forall a . a -> a)
+newtype Id = Id (forall a . a -> a)
 
 iD :: Id
 iD = Id (\ x -> x)
 
 main :: IO ()
 main = do
-  putStrLn $ show $ f id
-  putStrLn $ show $ g const
+  print $ f id
+  print $ g const
   case iD of
-    Id i -> putStrLn $ show (i (1::Int), i True)
+    Id i -> print (i (1::Int), i True)
