@@ -1,8 +1,6 @@
 module Serdes(main) where
-import Prelude
-import System.IO as IO
-import System.Environment
-import Debug.Trace
+
+import System.IO.Serialize
 
 f :: Int -> Int
 f x = x*2+1
@@ -12,4 +10,4 @@ main = do
   cprint ((+) :: Int->Int->Int)
   writeSerialized "f.tmp" f
   g <- readSerialized "f.tmp"
-  putStrLn $ show $ (g (5::Int) :: Int)
+  print (g (5::Int) :: Int)
